@@ -4,7 +4,8 @@ unsigned test(unsigned X) {
   return (unsigned char)X;
 }
 
-void main() {
+int main() {
 	printf("%d\n", test(123456));
+        return 0;
 }
 
