@@ -20,16 +20,34 @@
 #include "llvm/ADT/iterator"
 #include <stack>
 #include <set>
+#include <vector>
 
 namespace llvm {
 
-template<class GraphT, class GT = GraphTraits<GraphT> >
-class po_iterator : public forward_iterator<typename GT::NodeType, ptrdiff_t> {
+template<class SetType, bool External>   // Non-external set 
+class po_iterator_storage { 
+public: 
+  SetType Visited; 
+}; 
+
+template<class SetType> 
+class po_iterator_storage<SetType, true> { 
+public: 
+  po_iterator_storage(SetType &VSet) : Visited(VSet) {} 
+  po_iterator_storage(const po_iterator_storage &S) : Visited(S.Visited) {} 
+  SetType &Visited; 
+}; 
+
+template<class GraphT, 
+        class SetType = std::set<typename GraphTraits<GraphT>::NodeType*>, 
+        bool ExtStorage = false, 
+        class GT = GraphTraits<GraphT> > 
+class po_iterator : public forward_iterator<typename GT::NodeType, ptrdiff_t>, 
+                    public po_iterator_storage<SetType, ExtStorage> { 
   typedef forward_iterator<typename GT::NodeType, ptrdiff_t> super;
   typedef typename GT::NodeType          NodeType;
   typedef typename GT::ChildIteratorType ChildItTy;
-
-  std::set<NodeType *> Visited;    // All of the blocks visited so far...
+  
   // VisitStack - Used to maintain the ordering.  Top = current block
   // First element is basic block pointer, second is the 'next child' to visit
   std::stack<std::pair<NodeType *, ChildItTy> > VisitStack;
@@ -37,27 +55,45 @@ class po_iterator : public forward_iterator<typename GT::NodeType, ptrdiff_t> {
   void traverseChild() {
     while (VisitStack.top().second != GT::child_end(VisitStack.top().first)) {
       NodeType *BB = *VisitStack.top().second++;
-      if (!Visited.count(BB)) {  // If the block is not visited...
-        Visited.insert(BB);
+      if (!this->Visited.count(BB)) {  // If the block is not visited...
+        this->Visited.insert(BB);
         VisitStack.push(std::make_pair(BB, GT::child_begin(BB)));
       }
     }
   }
 
   inline po_iterator(NodeType *BB) {
-    Visited.insert(BB);
+    this->Visited.insert(BB);
     VisitStack.push(std::make_pair(BB, GT::child_begin(BB)));
     traverseChild();
   }
-  inline po_iterator() { /* End is when stack is empty */ }
+  inline po_iterator() {} // End is when stack is empty. 
+     
+  inline po_iterator(NodeType *BB, SetType &S) : 
+    po_iterator_storage<SetType, ExtStorage>(&S) { 
+    if(!S.count(BB)) { 
+      this->Visited.insert(BB); 
+      VisitStack.push(std::make_pair(BB, GT::child_begin(BB))); 
+      traverseChild(); 
+    } 
+  } 
+ 
+  inline po_iterator(SetType &S) : 
+      po_iterator_storage<SetType, ExtStorage>(&S) {
+  } // End is when stack is empty. 
 public:
   typedef typename super::pointer pointer;
-  typedef po_iterator<GraphT, GT> _Self;
+  typedef po_iterator<GraphT, SetType, ExtStorage, GT> _Self; 
 
   // Provide static "constructors"...
   static inline _Self begin(GraphT G) { return _Self(GT::getEntryNode(G)); }
   static inline _Self end  (GraphT G) { return _Self(); }
 
+  static inline _Self begin(GraphT G, SetType &S) { 
+    return _Self(GT::getEntryNode(G), S); 
+  } 
+  static inline _Self end  (GraphT G, SetType &S) { return _Self(S); } 
+  
   inline bool operator==(const _Self& x) const {
     return VisitStack == x.VisitStack;
   }
@@ -92,10 +128,30 @@ po_iterator<T> po_begin(T G) { return po_iterator<T>::begin(G); }
 template <class T>
 po_iterator<T> po_end  (T G) { return po_iterator<T>::end(G); }
 
+// Provide global definitions of external postorder iterators... 
+template<class T, class SetType=std::set<typename GraphTraits<T>::NodeType*> > 
+struct po_ext_iterator : public po_iterator<T, SetType, true> { 
+  po_ext_iterator(const po_iterator<T, SetType, true> &V) :  
+  po_iterator<T, SetType, true>(V) {} 
+}; 
+ 
+template<class T, class SetType> 
+po_ext_iterator<T, SetType> po_ext_begin(T G, SetType &S) { 
+  return po_ext_iterator<T, SetType>::begin(G, S); 
+} 
+
+template<class T, class SetType> 
+po_ext_iterator<T, SetType> po_ext_end(T G, SetType &S) { 
+  return po_ext_iterator<T, SetType>::end(G, S); 
+} 
+
 // Provide global definitions of inverse post order iterators...
-template <class T>
-struct ipo_iterator : public po_iterator<Inverse<T> > {
-  ipo_iterator(const po_iterator<Inverse<T> > &V) :po_iterator<Inverse<T> >(V){}
+template <class T, 
+          class SetType = std::set<typename GraphTraits<T>::NoddeType*>,  
+          bool External = false> 
+struct ipo_iterator : public po_iterator<Inverse<T>, SetType, External > { 
+  ipo_iterator(const po_iterator<Inverse<T>, SetType, External> &V) : 
+     po_iterator<Inverse<T>, SetType, External> (V) {} 
 };
 
 template <class T>
@@ -108,6 +164,24 @@ ipo_iterator<T> ipo_end(T G){
   return ipo_iterator<T>::end(G);
 }
 
+//Provide global definitions of external inverse postorder iterators... 
+template <class T, class SetType = std::set<typename GraphTraits<T>::NodeType*> > 
+struct ipo_ext_iterator : public ipo_iterator<T, SetType, true> { 
+  ipo_ext_iterator(const ipo_iterator<T, SetType, true> &V) : 
+    ipo_iterator<T, SetType, true>(&V) {} 
+  ipo_ext_iterator(const po_iterator<Inverse<T>, SetType, true> &V) : 
+    ipo_iterator<T, SetType, true>(&V) {} 
+}; 
+
+template <class T, class SetType> 
+ipo_ext_iterator<T, SetType> ipo_ext_begin(T G, SetType &S) { 
+  return ipo_ext_iterator<T, SetType>::begin(G, S); 
+} 
+
+template <class T, class SetType> 
+ipo_ext_iterator<T, SetType> ipo_ext_end(T G, SetType &S) { 
+  return ipo_ext_iterator<T, SetType>::end(G, S); 
+} 
 
 //===--------------------------------------------------------------------===//
 // Reverse Post Order CFG iterator code
