@@ -56,7 +56,7 @@ class ArchiveMember {
       CompressedBytecodeFlag = 32, ///< Member is compressed bytecode
       HasPathFlag = 64,            ///< Member has a full or partial path
       HasLongFilenameFlag = 128,   ///< Member uses the long filename syntax
-      StringTableFlag = 256,       ///< Member is an ar(1) format string table
+      StringTableFlag = 256        ///< Member is an ar(1) format string table
     };
 
   /// @}
@@ -489,6 +489,9 @@ class Archive {
     bool fillHeader(const ArchiveMember&mbr,
                     ArchiveMemberHeader& hdr,int sz, bool TruncateNames) const;
 
+    /// @brief Frees all the members and unmaps the archive file.
+    void cleanUpMemory();
+
     /// This type is used to keep track of bytecode modules loaded from the
     /// symbol table. It maps the file offset to a pair that consists of the
     /// associated ArchiveMember and the ModuleProvider.
