@@ -23,34 +23,47 @@
 #define LLVM_CODEGEN_MACHINECONSTANTPOOL_H
 
 #include <vector>
+#include <iosfwd>
 
 namespace llvm {
 
 class Constant;
+class TargetData;
 
+/// MachineConstantPoolEntry - One entry in the constant pool.
+///
+struct MachineConstantPoolEntry {
+  /// Val - The constant itself.
+  Constant *Val;
+  /// Offset - The offset of the constant from the start of the constant pool.
+  unsigned Offset;
+  
+  MachineConstantPoolEntry(Constant *V, unsigned O) : Val(V), Offset(O) {}
+};
+  
 class MachineConstantPool {
-  std::vector<Constant*> Constants;
+  const TargetData &TD;
+  unsigned PoolAlignment;
+  std::vector<MachineConstantPoolEntry> Constants;
 public:
-
+  MachineConstantPool(const TargetData &td) : TD(td), PoolAlignment(1) {}
+    
+  /// getConstantPoolAlignment - Return the log2 of the alignment required by
+  /// the whole constant pool, of which the first element must be aligned.
+  unsigned getConstantPoolAlignment() const { return PoolAlignment; }
+  
   /// getConstantPoolIndex - Create a new entry in the constant pool or return
-  /// an existing one.
+  /// an existing one.  User must specify an alignment in bytes for the object.
   ///
-  unsigned getConstantPoolIndex(Constant *C) {
-    // Check to see if we already have this constant.
-    //
-    // FIXME, this could be made much more efficient for large constant pools.
-    for (unsigned i = 0, e = Constants.size(); i != e; ++i)
-      if (Constants[i] == C)
-        return i;
-    Constants.push_back(C);
-    return Constants.size()-1;
-  }
-
+  unsigned getConstantPoolIndex(Constant *C, unsigned Alignment);
+  
   /// isEmpty - Return true if this constant pool contains no constants.
   ///
   bool isEmpty() const { return Constants.empty(); }
 
-  const std::vector<Constant*> &getConstants() const { return Constants; }
+  const std::vector<MachineConstantPoolEntry> &getConstants() const {
+    return Constants;
+  }
 
   /// print - Used by the MachineFunction printer to print information about
   /// stack objects.  Implemented in MachineFunction.cpp
@@ -58,6 +71,7 @@ public:
   void print(std::ostream &OS) const;
 
   /// dump - Call print(std::cerr) to be called from the debugger.
+  ///
   void dump() const;
 };
 
