@@ -19,6 +19,8 @@
 namespace llvm {
 
 class Constant : public User {
+  void operator=(const Constant &);     // Do not implement
+  Constant(const Constant &);           // Do not implement
 protected:
   Constant(const Type *Ty, ValueTy vty, Use *Ops, unsigned NumOps,
            const std::string& Name = "")
@@ -88,6 +90,13 @@ public:
   /// constant subsystem, which can be used in environments where this memory
   /// is otherwise reported as a leak.
   static void clearAllValueMaps();
+  
+  /// getStringValue - Turn an LLVM constant pointer that eventually points to a
+  /// global into a string value.  Return an empty string if we can't do it.
+  /// Parameter Chop determines if the result is chopped at the first null
+  /// terminator.
+  ///
+  std::string getStringValue(bool Chop = true, unsigned Offset = 0);
 };
 
 } // End llvm namespace
