@@ -15,6 +15,7 @@
 #define LLVM_TARGET_TARGETMACHINE_H
 
 #include "llvm/Target/TargetData.h"
+#include "llvm/Target/TargetInstrItineraries.h"
 #include <cassert>
 
 namespace llvm {
@@ -23,6 +24,7 @@ class TargetSubtarget;
 class TargetInstrInfo;
 class TargetInstrDescriptor;
 class TargetJITInfo;
+class TargetLowering;
 class TargetSchedInfo;
 class SparcV9RegInfo;
 class TargetFrameInfo;
@@ -31,7 +33,16 @@ class MRegisterInfo;
 class FunctionPassManager;
 class PassManager;
 class Pass;
-class IntrinsicLowering;
+
+// Relocation model types.
+namespace Reloc {
+  enum Model {
+    Default,
+    Static,
+    PIC,
+    DynamicNoPIC
+  };
+}
 
 //===----------------------------------------------------------------------===//
 ///
@@ -42,28 +53,24 @@ class IntrinsicLowering;
 class TargetMachine {
   const std::string Name;
   const TargetData DataLayout;       // Calculates type size & alignment
-  IntrinsicLowering *IL;             // Specifies how to lower intrinsic calls
 
   TargetMachine(const TargetMachine&);   // DO NOT IMPLEMENT
   void operator=(const TargetMachine&);  // DO NOT IMPLEMENT
 protected: // Can only create subclasses...
-  TargetMachine(const std::string &name, IntrinsicLowering *IL,
-                bool LittleEndian = false,
+  TargetMachine(const std::string &name, bool LittleEndian = false,
                 unsigned char PtrSize = 8, unsigned char PtrAl = 8,
                 unsigned char DoubleAl = 8, unsigned char FloatAl = 4,
                 unsigned char LongAl = 8, unsigned char IntAl = 4,
                 unsigned char ShortAl = 2, unsigned char ByteAl = 1,
                 unsigned char BoolAl = 1);
 
-  TargetMachine(const std::string &name, IntrinsicLowering *IL,
-                const TargetData &TD);
+  TargetMachine(const std::string &name, const TargetData &TD);
 
   /// This constructor is used for targets that support arbitrary TargetData
   /// layouts, like the C backend.  It initializes the TargetData to match that
   /// of the specified module.
   ///
-  TargetMachine(const std::string &name, IntrinsicLowering *IL,
-                const Module &M);
+  TargetMachine(const std::string &name, const Module &M);
 
   /// getSubtargetImpl - virtual method implemented by subclasses that returns
   /// a reference to that target's TargetSubtarget-derived member variable.
@@ -87,19 +94,15 @@ public:
 
   const std::string &getName() const { return Name; }
 
-  /// getIntrinsicLowering - This method returns a reference to an
-  /// IntrinsicLowering instance which should be used by the code generator to
-  /// lower unknown intrinsic functions to the equivalent LLVM expansion.
-  ///
-  IntrinsicLowering &getIntrinsicLowering() const { return *IL; }
-
   // Interfaces to the major aspects of target machine information:
   // -- Instruction opcode and operand information
   // -- Pipelines and scheduling information
   // -- Stack frame information
+  // -- Selection DAG lowering information
   //
   virtual const TargetInstrInfo        *getInstrInfo() const { return 0; }
   virtual const TargetFrameInfo        *getFrameInfo() const { return 0; }
+  virtual       TargetLowering    *getTargetLowering() const { return 0; }
   const TargetData &getTargetData() const { return DataLayout; }
 
   /// getSubtarget - This method returns a pointer to the specified type of
@@ -122,11 +125,25 @@ public:
   /// otherwise return null.
   ///
   virtual TargetJITInfo *getJITInfo() { return 0; }
+  
+  /// getInstrItineraryData - Returns instruction itinerary data for the target
+  /// or specific subtarget.
+  ///
+  virtual const InstrItineraryData getInstrItineraryData() const {  
+    return InstrItineraryData();
+  }
 
   // These are deprecated interfaces.
   virtual const TargetSchedInfo        *getSchedInfo() const { return 0; }
   virtual const SparcV9RegInfo         *getRegInfo()   const { return 0; }
 
+  /// getRelocationModel - Returns the code generation relocation model. The
+  /// choices are static, PIC, and dynamic-no-pic, and target default.
+  static Reloc::Model getRelocationModel();
+
+  /// setRelocationModel - Sets the code generation relocation model.
+  static void setRelocationModel(Reloc::Model Model);
+
   /// CodeGenFileType - These enums are meant to be passed into
   /// addPassesToEmitFile to indicate what type of file to emit.
   enum CodeGenFileType {
@@ -135,11 +152,12 @@ public:
 
   /// addPassesToEmitFile - Add passes to the specified pass manager to get
   /// the specified file emitted.  Typically this will involve several steps of
-  /// code generation.  This method should return true if emission of this file
-  /// type is not supported.
+  /// code generation.  If Fast is set to true, the code generator should emit
+  /// code as fast as possible, without regard for compile time.  This method
+  /// should return true if emission of this file type is not supported.
   ///
   virtual bool addPassesToEmitFile(PassManager &PM, std::ostream &Out,
-                                   CodeGenFileType FileType) {
+                                   CodeGenFileType FileType, bool Fast) {
     return true;
   }
 
