@@ -15,6 +15,8 @@
 #ifndef LLVM_TRANSFORMS_SCALAR_H
 #define LLVM_TRANSFORMS_SCALAR_H
 
+#include <cstdlib>
+
 namespace llvm {
 
 class ModulePass;
@@ -22,6 +24,7 @@ class FunctionPass;
 class GetElementPtrInst;
 class PassInfo;
 class TerminatorInst;
+class TargetLowering;
 
 //===----------------------------------------------------------------------===//
 //
@@ -129,12 +132,11 @@ FunctionPass *createLICMPass();
 //===----------------------------------------------------------------------===//
 //
 // LoopStrengthReduce - This pass is strength reduces GEP instructions that use
-// a loop's canonical induction variable as one of their indices.  The
-// MaxTargetAMSize is the largest element size that the target architecture
-// can handle in its addressing modes.  Power of two multipliers less than or
-// equal to this value are not reduced.
+// a loop's canonical induction variable as one of their indices.  It takes an
+// optional parameter used to consult the target machine whether certain
+// transformations are profitable.
 //
-FunctionPass *createLoopStrengthReducePass(unsigned MaxTargetAMSize = 1);
+FunctionPass *createLoopStrengthReducePass(const TargetLowering *TLI = NULL);
 
 //===----------------------------------------------------------------------===//
 //
@@ -162,6 +164,13 @@ FunctionPass *createLoopUnrollPass();
 //
 FunctionPass *createPromoteMemoryToRegisterPass();
 
+//===----------------------------------------------------------------------===//
+//
+// This pass is used to demote registers to memory references .
+// In basically undoes the PromoteMemoryToRegister pass to
+// make cfg hacking easier.
+FunctionPass *createDemoteRegisterToMemoryPass();
+extern const PassInfo *DemoteRegisterToMemoryID;
 
 //===----------------------------------------------------------------------===//
 //
@@ -271,7 +280,8 @@ FunctionPass *createLowerPackedPass();
 // "my LLVM-to-LLVM pass doesn't support the invoke instruction yet" lowering
 // pass.
 //
-FunctionPass *createLowerInvokePass();
+FunctionPass *createLowerInvokePass(unsigned JumBufSize = 200, 
+                                    unsigned JumpBufAlign = 0);
 extern const PassInfo *LowerInvokePassID;
 
 
