@@ -24,6 +24,7 @@
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"
 #include "llvm/Support/Timer.h"
+#include <iostream>
 
 // FIXME: This should eventually be a FunctionPass that is automatically
 // aggregated into a Pass.
@@ -39,6 +40,18 @@ static cl::opt<bool>
 TrackIntegersAsPointers("dsa-track-integers", cl::Hidden,
          cl::desc("If this is set, track integers as potential pointers"));
 
+static cl::list<std::string>
+AllocList("dsa-alloc-list",
+          cl::value_desc("list"),
+          cl::desc("List of functions that allocate memory from the heap"),
+          cl::CommaSeparated, cl::Hidden);
+
+static cl::list<std::string>
+FreeList("dsa-free-list",
+          cl::value_desc("list"),
+          cl::desc("List of functions that free memory from the heap"),
+          cl::CommaSeparated, cl::Hidden);
+
 namespace llvm {
 namespace DS {
   // isPointerType - Return true if this type is big enough to hold a pointer.
@@ -434,7 +447,24 @@ void GraphBuilder::visitGetElementPtrInst(User &GEP) {
   // Add in the offset calculated...
   Value.setOffset(Value.getOffset()+Offset);
 
-  // Value is now the pointer we want to GEP to be...
+  // Check the offset
+  DSNode *N = Value.getNode();
+  if (N &&
+      !N->isNodeCompletelyFolded() &&
+      (N->getSize() != 0 || Offset != 0) &&
+      !N->isForwarding()) {
+    if ((Offset >= N->getSize()) || int(Offset) < 0) {
+      // Accessing offsets out of node size range
+      // This is seen in the "magic" struct in named (from bind), where the
+      // fourth field is an array of length 0, presumably used to create struct
+      // instances of different sizes
+
+      // Collapse the node since its size is now variable
+      N->foldNodeCompletely();
+    }
+  }
+  
+  // Value is now the pointer we want to GEP to be...  
   setDestTo(GEP, Value);
 }
 
@@ -515,8 +545,16 @@ void GraphBuilder::visitCallSite(CallSite CS) {
         return;
       case Intrinsic::vaend:
         return;  // noop
-      case Intrinsic::memmove:
-      case Intrinsic::memcpy: {
+      case Intrinsic::memcpy_i32: 
+      case Intrinsic::memcpy_i64:
+      case Intrinsic::memmove_i32:
+      case Intrinsic::memmove_i64: {
+        //This is over aggressive.  What these functions do is not make the 
+        // targets pointers alias, but rather merge the out edges of the graphs
+        // for the pointers according to the type merging of the graphs.
+        //Simply merging the two graphs is a crude approximation to this.
+        //I might be wrong though.
+
         // Merge the first & second arguments, and mark the memory read and
         // modified.
         DSNodeHandle RetNH = getValueDest(**CS.arg_begin());
@@ -525,12 +563,40 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           N->setModifiedMarker()->setReadMarker();
         return;
       }
-      case Intrinsic::memset:
+      case Intrinsic::memset_i32:
+      case Intrinsic::memset_i64:
         // Mark the memory modified.
         if (DSNode *N = getValueDest(**CS.arg_begin()).getNode())
           N->setModifiedMarker();
         return;
       default:
+        // Determine if the called function is one of the specified heap
+        // allocation functions
+        for (cl::list<std::string>::iterator AllocFunc = AllocList.begin(),
+             LastAllocFunc = AllocList.end();
+             AllocFunc != LastAllocFunc;
+             ++AllocFunc) {
+          if (F->getName() == *(AllocFunc)) {
+            setDestTo(*CS.getInstruction(),
+                      createNode()->setHeapNodeMarker()->setModifiedMarker());
+            return;
+          }
+        }
+
+        // Determine if the called function is one of the specified heap
+        // free functions
+        for (cl::list<std::string>::iterator FreeFunc = FreeList.begin(),
+             LastFreeFunc = FreeList.end();
+             FreeFunc != LastFreeFunc;
+             ++FreeFunc) {
+          if (F->getName() == *(FreeFunc)) {
+            // Mark that the node is written to...
+            if (DSNode *N = getValueDest(*(CS.getArgument(0))).getNode())
+              N->setModifiedMarker()->setHeapNodeMarker();
+            return;
+          }
+        }
+
         if (F->getName() == "calloc" || F->getName() == "posix_memalign" ||
             F->getName() == "memalign" || F->getName() == "valloc") {
           setDestTo(*CS.getInstruction(),
@@ -616,7 +682,6 @@ void GraphBuilder::visitCallSite(CallSite CS) {
               Link.mergeWith(getValueDest(**CS.arg_begin()));
             }
           }
-
           return;
         } else if (F->getName() == "fopen" || F->getName() == "fdopen" ||
                    F->getName() == "freopen") {
@@ -921,6 +986,17 @@ void GraphBuilder::visitCallSite(CallSite CS) {
             N->mergeTypeInfo(Type::DoubleTy, H.getOffset());
           }
           return;
+        } else if (F->getName() == "strcat" || F->getName() == "strncat") {
+          //This might be making unsafe assumptions about usage
+          //Merge return and first arg
+          DSNodeHandle RetNH = getValueDest(*CS.getInstruction());
+          RetNH.mergeWith(getValueDest(**CS.arg_begin()));
+          if (DSNode *N = RetNH.getNode())
+            N->setHeapNodeMarker()->setModifiedMarker()->setReadMarker();
+          //and read second pointer
+          if (DSNode *N = getValueDest(**(CS.arg_begin() + 1)).getNode())
+            N->setReadMarker();
+          return;
         } else {
           // Unknown function, warn if it returns a pointer type or takes a
           // pointer argument.
@@ -1040,8 +1116,17 @@ void GraphBuilder::MergeConstantInitIntoNode(DSNodeHandle &NH, Constant *C) {
     const StructLayout *SL = TD.getStructLayout(CS->getType());
     for (unsigned i = 0, e = CS->getNumOperands(); i != e; ++i) {
       DSNode *NHN = NH.getNode();
-      DSNodeHandle NewNH(NHN, NH.getOffset()+(unsigned)SL->MemberOffsets[i]);
-      MergeConstantInitIntoNode(NewNH, cast<Constant>(CS->getOperand(i)));
+      //Some programmers think ending a structure with a [0 x sbyte] is cute
+      //This should be ok as the allocation type should grow this type when
+      //it is merged in if it is bigger.
+      if (SL->MemberOffsets[i] < SL->StructSize) {
+        DSNodeHandle NewNH(NHN, NH.getOffset()+(unsigned)SL->MemberOffsets[i]);
+        MergeConstantInitIntoNode(NewNH, cast<Constant>(CS->getOperand(i)));
+      } else if (SL->MemberOffsets[i] == SL->StructSize) {
+        DEBUG(std::cerr << "Zero size element at end of struct\n");
+      } else {
+        assert(0 && "type was smaller than offsets of of struct layout indicate");
+      }
     }
   } else if (isa<ConstantAggregateZero>(C) || isa<UndefValue>(C)) {
     // Noop
