@@ -80,8 +80,9 @@ namespace {
     void visitGetElementPtrInst(GetElementPtrInst &I);
     void visitCastInst(CastInst &I);
     void visitShiftInst(ShiftInst &I) { handleBinaryInst((Instruction&)I); }
+    void visitSelectInst(SelectInst &I);
     void visitInstruction(Instruction &) {
-      // Cannot value number calls or terminator instructions...
+      // Cannot value number calls or terminator instructions.
     }
   };
 }
@@ -198,4 +199,38 @@ void BVNImpl::visitGetElementPtrInst(GetElementPtrInst &I) {
       }
 }
 
+// isIdenticalSelectInst - Return true if the two select instructions are
+// identical.
+//
+static inline bool isIdenticalSelectInst(const SelectInst &I1,
+                                         const SelectInst *I2) {
+  // Is it embedded in the same function?  (This could be false if LHS
+  // is a constant or global!)
+  if (I1.getParent()->getParent() != I2->getParent()->getParent())
+    return false;
+  
+  // They are identical if both operands are the same!
+  return I1.getOperand(0) == I2->getOperand(0) &&
+         I1.getOperand(1) == I2->getOperand(1) &&
+         I1.getOperand(2) == I2->getOperand(2);
+    return true;
+  
+  return false;
+}
+
+void BVNImpl::visitSelectInst(SelectInst &I) {
+  Value *Cond = I.getOperand(0);
+  
+  for (Value::use_iterator UI = Cond->use_begin(), UE = Cond->use_end();
+       UI != UE; ++UI)
+    if (SelectInst *Other = dyn_cast<SelectInst>(*UI))
+      // Check to see if this new select is not I, but has the same operands.
+      if (Other != &I && isIdenticalSelectInst(I, Other)) {
+        // These instructions are identical.  Handle the situation.
+        RetVals.push_back(Other);
+      }
+        
+}
+
+
 void llvm::BasicValueNumberingStub() { }
