@@ -17,11 +17,28 @@
 #include "llvm/CodeGen/IntrinsicLowering.h"
 #include "llvm/DerivedTypes.h"
 #include "llvm/Module.h"
+#include "llvm/ModuleProvider.h"
 using namespace llvm;
 
+static struct RegisterInterp {
+  RegisterInterp() { Interpreter::Register(); }
+} InterpRegistrator;
+
+namespace llvm {
+  void LinkInInterpreter() {
+  }
+}
+
 /// create - Create a new interpreter object.  This can never fail.
 ///
-ExecutionEngine *Interpreter::create(Module *M, IntrinsicLowering *IL) {
+ExecutionEngine *Interpreter::create(ModuleProvider *MP) {
+  Module *M;
+  try {
+    M = MP->materializeModule();
+  } catch (...) {
+    return 0;  // error materializing the module.
+  }
+  
   bool isLittleEndian = false;
   switch (M->getEndianness()) {
   case Module::LittleEndian: isLittleEndian = true; break;
@@ -42,25 +59,25 @@ ExecutionEngine *Interpreter::create(Module *M, IntrinsicLowering *IL) {
     break;
   }
 
-  return new Interpreter(M, isLittleEndian, isLongPointer, IL);
+  return new Interpreter(M, isLittleEndian, isLongPointer);
 }
 
 //===----------------------------------------------------------------------===//
 // Interpreter ctor - Initialize stuff
 //
-Interpreter::Interpreter(Module *M, bool isLittleEndian, bool isLongPointer,
-                         IntrinsicLowering *il)
-  : ExecutionEngine(M), ExitCode(0),
+Interpreter::Interpreter(Module *M, bool isLittleEndian, bool isLongPointer)
+  : ExecutionEngine(M),
     TD("lli", isLittleEndian, isLongPointer ? 8 : 4, isLongPointer ? 8 : 4,
-       isLongPointer ? 8 : 4), IL(il) {
+       isLongPointer ? 8 : 4) {
 
+  memset(&ExitValue, 0, sizeof(ExitValue));
   setTargetData(TD);
   // Initialize the "backend"
   initializeExecutionEngine();
   initializeExternalFunctions();
   emitGlobals();
 
-  if (IL == 0) IL = new DefaultIntrinsicLowering();
+  IL = new DefaultIntrinsicLowering();
 }
 
 Interpreter::~Interpreter() {
@@ -100,8 +117,6 @@ Interpreter::runFunction(Function *F,
   // Start executing the function.
   run();
 
-  GenericValue rv;
-  rv.IntVal = ExitCode;
-  return rv;
+  return ExitValue;
 }
 
