@@ -26,8 +26,9 @@
 #include "llvm/Support/Debug.h"
 #include "llvm/ADT/Statistic.h"
 #include "llvm/System/Memory.h"
-#include <list>
 #include <algorithm>
+#include <iostream>
+#include <list>
 using namespace llvm;
 
 namespace {
@@ -105,8 +106,8 @@ JITMemoryManager::JITMemoryManager(bool useGOT) {
 }
 
 JITMemoryManager::~JITMemoryManager() {
-  for (std::list<sys::MemoryBlock>::iterator ib = Blocks.begin(), ie = Blocks.end();
-       ib != ie; ++ib)
+  for (std::list<sys::MemoryBlock>::iterator ib = Blocks.begin(),
+       ie = Blocks.end(); ib != ie; ++ib)
     sys::Memory::ReleaseRWX(*ib);
   Blocks.clear();
 }
@@ -352,14 +353,15 @@ void *JITResolver::JITCompilerFn(void *Stub) {
   // a little bit after the stub.  As such, use upper_bound to find it.
   std::map<void*, Function*>::iterator I =
     JR.state.getStubToFunctionMap(locked).upper_bound(Stub);
-  assert(I != JR.state.getStubToFunctionMap(locked).begin() && "This is not a known stub!");
+  assert(I != JR.state.getStubToFunctionMap(locked).begin() &&
+         "This is not a known stub!");
   Function *F = (--I)->second;
 
   // We might like to remove the stub from the StubToFunction map.
   // We can't do that! Multiple threads could be stuck, waiting to acquire the
   // lock above. As soon as the 1st function finishes compiling the function,
-  // the next one will be released, and needs to be able to find the function it needs
-  // to call.
+  // the next one will be released, and needs to be able to find the function it
+  // needs to call.
   //JR.state.getStubToFunctionMap(locked).erase(I);
 
   DEBUG(std::cerr << "JIT: Lazily resolving function '" << F->getName()
@@ -417,18 +419,19 @@ namespace {
     // save CurBlock and CurByte here.
     unsigned char *SavedCurBlock, *SavedCurByte;
 
-    // ConstantPoolAddresses - Contains the location for each entry in the
-    // constant pool.
-    std::vector<void*> ConstantPoolAddresses;
-
     /// Relocations - These are the relocations that the function needs, as
     /// emitted.
     std::vector<MachineRelocation> Relocations;
 
+    /// ConstantPool - The constant pool for the current function.
+    ///
+    MachineConstantPool *ConstantPool;
+
+    /// ConstantPoolBase - A pointer to the first entry in the constant pool.
+    ///
+    void *ConstantPoolBase;
   public:
-    JITEmitter(JIT &jit)
-      :MemMgr(jit.getJITInfo().needsGOT())
-    {
+    JITEmitter(JIT &jit) : MemMgr(jit.getJITInfo().needsGOT()) {
       TheJIT = &jit;
       DEBUG(std::cerr <<
             (MemMgr.isManagingGOT() ? "JIT is managing GOT\n"
@@ -534,7 +537,8 @@ void JITEmitter::finishFunction(MachineFunction &F) {
         MR.setGOTIndex(idx);
         if (((void**)MemMgr.getGOTBase())[idx] != ResultPtr) {
           DEBUG(std::cerr << "GOT was out of date for " << ResultPtr
-                << " pointing at " << ((void**)MemMgr.getGOTBase())[idx] << "\n");
+                << " pointing at " << ((void**)MemMgr.getGOTBase())[idx]
+                << "\n");
           ((void**)MemMgr.getGOTBase())[idx] = ResultPtr;
         }
       }
@@ -559,21 +563,23 @@ void JITEmitter::finishFunction(MachineFunction &F) {
                   << ": " << CurByte-CurBlock << " bytes of text, "
                   << Relocations.size() << " relocations\n");
   Relocations.clear();
-  ConstantPoolAddresses.clear();
 }
 
 void JITEmitter::emitConstantPool(MachineConstantPool *MCP) {
-  const std::vector<Constant*> &Constants = MCP->getConstants();
+  const std::vector<MachineConstantPoolEntry> &Constants = MCP->getConstants();
   if (Constants.empty()) return;
 
-  for (unsigned i = 0, e = Constants.size(); i != e; ++i) {
-    const Type *Ty = Constants[i]->getType();
-    unsigned Size      = (unsigned)TheJIT->getTargetData().getTypeSize(Ty);
-    unsigned Alignment = TheJIT->getTargetData().getTypeAlignment(Ty);
+  unsigned Size = Constants.back().Offset;
+  Size += TheJIT->getTargetData().getTypeSize(Constants.back().Val->getType());
 
-    void *Addr = MemMgr.allocateConstant(Size, Alignment);
-    TheJIT->InitializeMemory(Constants[i], Addr);
-    ConstantPoolAddresses.push_back(Addr);
+  ConstantPoolBase = MemMgr.allocateConstant(Size, 
+                                       1 << MCP->getConstantPoolAlignment());
+  ConstantPool = MCP;
+  
+  // Initialize the memory for all of the constant pool entries.
+  for (unsigned i = 0, e = Constants.size(); i != e; ++i) {
+    void *CAddr = (char*)ConstantPoolBase+Constants[i].Offset;
+    TheJIT->InitializeMemory(Constants[i].Val, CAddr);
   }
 }
 
@@ -609,9 +615,10 @@ void JITEmitter::emitWordAt(unsigned W, unsigned *Ptr) {
 // method.
 //
 uint64_t JITEmitter::getConstantPoolEntryAddress(unsigned ConstantNum) {
-  assert(ConstantNum < ConstantPoolAddresses.size() &&
+  assert(ConstantNum < ConstantPool->getConstants().size() &&
          "Invalid ConstantPoolIndex!");
-  return (intptr_t)ConstantPoolAddresses[ConstantNum];
+  return (intptr_t)ConstantPoolBase +
+         ConstantPool->getConstants()[ConstantNum].Offset;
 }
 
 unsigned char* JITEmitter::allocateGlobal(unsigned size, unsigned alignment)
