@@ -290,6 +290,11 @@ static Value *RemapOperand(const Value *In,
       Result = const_cast<Constant*>(CPV);
     } else if (isa<GlobalValue>(CPV)) {
       Result = cast<Constant>(RemapOperand(CPV, ValueMap));
+    } else if (const ConstantPacked *CP = dyn_cast<ConstantPacked>(CPV)) {
+      std::vector<Constant*> Operands(CP->getNumOperands());
+      for (unsigned i = 0, e = CP->getNumOperands(); i != e; ++i)
+        Operands[i] = cast<Constant>(RemapOperand(CP->getOperand(i), ValueMap));
+      Result = ConstantPacked::get(Operands);
     } else if (const ConstantExpr *CE = dyn_cast<ConstantExpr>(CPV)) {
       if (CE->getOpcode() == Instruction::GetElementPtr) {
         Value *Ptr = RemapOperand(CE->getOperand(0), ValueMap);
@@ -300,6 +305,24 @@ static Value *RemapOperand(const Value *In,
                                                         ValueMap)));
 
         Result = ConstantExpr::getGetElementPtr(cast<Constant>(Ptr), Indices);
+      } else if (CE->getOpcode() == Instruction::ExtractElement) {
+        Value *Ptr = RemapOperand(CE->getOperand(0), ValueMap);
+        Value *Idx = RemapOperand(CE->getOperand(1), ValueMap);
+        Result = ConstantExpr::getExtractElement(cast<Constant>(Ptr),
+                                                 cast<Constant>(Idx));
+      } else if (CE->getOpcode() == Instruction::InsertElement) {
+        Value *Ptr = RemapOperand(CE->getOperand(0), ValueMap);
+        Value *Elt = RemapOperand(CE->getOperand(1), ValueMap);
+        Value *Idx = RemapOperand(CE->getOperand(2), ValueMap);
+        Result = ConstantExpr::getInsertElement(cast<Constant>(Ptr),
+                                                cast<Constant>(Elt),
+                                                cast<Constant>(Idx));
+      } else if (CE->getOpcode() == Instruction::ShuffleVector) {
+        Value *V1 = RemapOperand(CE->getOperand(0), ValueMap);
+        Value *V2 = RemapOperand(CE->getOperand(1), ValueMap);
+        Result = ConstantExpr::getShuffleVector(cast<Constant>(V1),
+                                                cast<Constant>(V2),
+                                             cast<Constant>(CE->getOperand(2)));
       } else if (CE->getNumOperands() == 1) {
         // Cast instruction
         assert(CE->getOpcode() == Instruction::Cast);
@@ -748,6 +771,8 @@ static bool LinkAppendingVars(Module *M,
       unsigned NewSize = T1->getNumElements() + T2->getNumElements();
       ArrayType *NewType = ArrayType::get(T1->getElementType(), NewSize);
 
+      G1->setName("");   // Clear G1's name in case of a conflict!
+      
       // Create the new global variable...
       GlobalVariable *NG =
         new GlobalVariable(NewType, G1->isConstant(), G1->getLinkage(),
@@ -826,6 +851,14 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
       Dest->getTargetTriple() != Src->getTargetTriple())
     std::cerr << "WARNING: Linking two modules of different target triples!\n";
 
+  if (!Src->getModuleInlineAsm().empty()) {
+    if (Dest->getModuleInlineAsm().empty())
+      Dest->setModuleInlineAsm(Src->getModuleInlineAsm());
+    else
+      Dest->setModuleInlineAsm(Dest->getModuleInlineAsm()+"\n"+
+                               Src->getModuleInlineAsm());
+  }
+  
   // Update the destination module's dependent libraries list with the libraries
   // from the source module. There's no opportunity for duplicates here as the
   // Module ensures that duplicate insertions are discarded.
