@@ -15,13 +15,21 @@
 #include "llvm/Support/PluginLoader.h"
 #include "llvm/System/DynamicLibrary.h"
 #include <iostream>
+#include <vector>
 
 using namespace llvm;
 
+static std::vector<std::string>* plugins;
+
 void PluginLoader::operator=(const std::string &Filename) {
   std::string ErrorMessage;
+
+  if (!plugins)
+    plugins = new std::vector<std::string>();
+
   try {
     sys::DynamicLibrary::LoadLibraryPermanently(Filename.c_str());
+    plugins->push_back(Filename);
   } catch (const std::string& errmsg) {
     if (errmsg.empty()) {
       ErrorMessage = "Unknown";
@@ -33,3 +41,17 @@ void PluginLoader::operator=(const std::string &Filename) {
     std::cerr << "Error opening '" << Filename << "': " << ErrorMessage
               << "\n  -load request ignored.\n";
 }
+
+unsigned PluginLoader::getNumPlugins()
+{
+  if(plugins)
+    return plugins->size();
+  else 
+    return 0;
+}
+
+std::string& PluginLoader::getPlugin(unsigned num)
+{
+  assert(plugins && num < plugins->size() && "Asking for an out of bounds plugin");
+  return (*plugins)[num];
+}
