@@ -12,63 +12,25 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/Support/SlowOperationInformer.h"
-#include "llvm/Config/config.h"     // Get the signal handler return type
+#include "llvm/System/Alarm.h"
 #include <iostream>
 #include <sstream>
-#include <signal.h>
-#include <unistd.h>
 #include <cassert>
 using namespace llvm;
 
-/// OperationCancelled - This flag is set by the SIGINT signal handler if the
-/// user presses CTRL-C.
-static volatile bool OperationCancelled;
-
-/// ShouldShowStatus - This flag gets set if the operation takes a long time.
-///
-static volatile bool ShouldShowStatus;
-
-/// NestedSOI - Sanity check.  SlowOperationInformers cannot be nested or run in
-/// parallel.  This ensures that they never do.
-static bool NestedSOI = false;
-
-static RETSIGTYPE SigIntHandler(int Sig) {
-  OperationCancelled = true;
-  signal(SIGINT, SigIntHandler);
-}
-
-static RETSIGTYPE SigAlarmHandler(int Sig) {
-  ShouldShowStatus = true;
-}
-
-static void (*OldSigIntHandler) (int);
-
-
 SlowOperationInformer::SlowOperationInformer(const std::string &Name)
   : OperationName(Name), LastPrintAmount(0) {
-  assert(!NestedSOI && "SlowerOperationInformer objects cannot be nested!");
-  NestedSOI = true;
-
-  OperationCancelled = 0;
-  ShouldShowStatus = 0;
-
-  signal(SIGALRM, SigAlarmHandler);
-  OldSigIntHandler = signal(SIGINT, SigIntHandler);
-  alarm(1);
+  sys::SetupAlarm(1);
 }
 
 SlowOperationInformer::~SlowOperationInformer() {
-  NestedSOI = false;
+  sys::TerminateAlarm();
   if (LastPrintAmount) {
     // If we have printed something, make _sure_ we print the 100% amount, and
     // also print a newline.
     std::cout << std::string(LastPrintAmount, '\b') << "Progress "
               << OperationName << ": 100%  \n";
   }
-
-  alarm(0);
-  signal(SIGALRM, SIG_DFL);
-  signal(SIGINT, OldSigIntHandler);
 }
 
 /// progress - Clients should periodically call this method when they are in
@@ -76,7 +38,8 @@ SlowOperationInformer::~SlowOperationInformer() {
 /// along the operation is, given in 1/10ths of a percent (in other words,
 /// Amount should range from 0 to 1000).
 void SlowOperationInformer::progress(unsigned Amount) {
-  if (OperationCancelled) {
+  int status = sys::AlarmStatus();
+  if (status == -1) {
     std::cout << "\n";
     LastPrintAmount = 0;
     throw "While " + OperationName + ", operation cancelled.";
@@ -84,7 +47,7 @@ void SlowOperationInformer::progress(unsigned Amount) {
 
   // If we haven't spent enough time in this operation to warrant displaying the
   // progress bar, don't do so yet.
-  if (!ShouldShowStatus)
+  if (status == 0)
     return;
 
   // Delete whatever we printed last time.
