@@ -21,6 +21,7 @@
 #include "llvm/CodeGen/MachineInstrBuilder.h"
 #include "llvm/CodeGen/MachineFunction.h"
 #include "llvm/CodeGen/MachineFrameInfo.h"
+#include "llvm/CodeGen/MachineLocation.h"
 #include "llvm/Target/TargetFrameInfo.h"
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Target/TargetOptions.h"
@@ -56,11 +57,13 @@ void X86RegisterInfo::storeRegToStackSlot(MachineBasicBlock &MBB,
   } else if (RC == &X86::R16RegClass) {
     Opc = X86::MOV16mr;
   } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
-    Opc = X86::FST64m;
-  } else if (RC == &X86::V4F4RegClass) {
+    Opc = X86::FpST64m;
+  } else if (RC == &X86::FR32RegClass) {
     Opc = X86::MOVSSmr;
-  } else if (RC == &X86::V2F8RegClass) {
+  } else if (RC == &X86::FR64RegClass) {
     Opc = X86::MOVSDmr;
+  } else if (RC == &X86::VR128RegClass) {
+    Opc = X86::MOVAPDmr;
   } else {
     assert(0 && "Unknown regclass");
     abort();
@@ -80,11 +83,13 @@ void X86RegisterInfo::loadRegFromStackSlot(MachineBasicBlock &MBB,
   } else if (RC == &X86::R16RegClass) {
     Opc = X86::MOV16rm;
   } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
-    Opc = X86::FLD64m;
-  } else if (RC == &X86::V4F4RegClass) {
+    Opc = X86::FpLD64m;
+  } else if (RC == &X86::FR32RegClass) {
     Opc = X86::MOVSSrm;
-  } else if (RC == &X86::V2F8RegClass) {
+  } else if (RC == &X86::FR64RegClass) {
     Opc = X86::MOVSDrm;
+  } else if (RC == &X86::VR128RegClass) {
+    Opc = X86::MOVAPDrm;
   } else {
     assert(0 && "Unknown regclass");
     abort();
@@ -105,10 +110,12 @@ void X86RegisterInfo::copyRegToReg(MachineBasicBlock &MBB,
     Opc = X86::MOV16rr;
   } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
     Opc = X86::FpMOV;
-  } else if (RC == &X86::V4F4RegClass) {
-    Opc = X86::MOVSSrr;
-  } else if (RC == &X86::V2F8RegClass) {
-    Opc = X86::MOVSDrr;
+  } else if (RC == &X86::FR32RegClass) {
+    Opc = X86::FsMOVAPSrr;
+  } else if (RC == &X86::FR64RegClass) {
+    Opc = X86::FsMOVAPDrr;
+  } else if (RC == &X86::VR128RegClass) {
+    Opc = X86::MOVAPSrr;
   } else {
     assert(0 && "Unknown regclass");
     abort();
@@ -116,30 +123,6 @@ void X86RegisterInfo::copyRegToReg(MachineBasicBlock &MBB,
   BuildMI(MBB, MI, Opc, 1, DestReg).addReg(SrcReg);
 }
 
-unsigned X86RegisterInfo::isLoadFromStackSlot(MachineInstr *MI, 
-                                              int &FrameIndex) const {
-  switch (MI->getOpcode()) {
-  default: break;
-  case X86::MOV8rm:
-  case X86::MOV16rm:
-  case X86::MOV32rm:
-  case X86::FLD64m:
-  case X86::FLD80m:
-  case X86::MOVSSrm:
-  case X86::MOVSDrm:
-    if (MI->getOperand(1).isFrameIndex() && MI->getOperand(2).isImmediate() &&
-        MI->getOperand(3).isRegister() && MI->getOperand(4).isImmediate() &&
-        MI->getOperand(2).getImmedValue() == 1 &&
-        MI->getOperand(3).getReg() == 0 &&
-        MI->getOperand(4).getImmedValue() == 0) {
-      FrameIndex = MI->getOperand(1).getFrameIndex();
-      return MI->getOperand(0).getReg();
-    }
-    break;
-  }
-  return 0;
-}
-
 
 static MachineInstr *MakeMInst(unsigned Opcode, unsigned FrameIndex,
                                MachineInstr *MI) {
@@ -166,11 +149,17 @@ static MachineInstr *MakeMIInst(unsigned Opcode, unsigned FrameIndex,
       .addZImm(MI->getOperand(1).getImmedValue());
   else if (MI->getOperand(1).isGlobalAddress())
     return addFrameReference(BuildMI(Opcode, 5), FrameIndex)
-      .addGlobalAddress(MI->getOperand(1).getGlobal());
+      .addGlobalAddress(MI->getOperand(1).getGlobal(),
+                        false, MI->getOperand(1).getOffset());
   assert(0 && "Unknown operand for MakeMI!");
   return 0;
 }
 
+static MachineInstr *MakeM0Inst(unsigned Opcode, unsigned FrameIndex,
+                                MachineInstr *MI) {
+  return addFrameReference(BuildMI(Opcode, 5), FrameIndex).addZImm(0);
+}
+
 static MachineInstr *MakeRMInst(unsigned Opcode, unsigned FrameIndex,
                                 MachineInstr *MI) {
   const MachineOperand& op = MI->getOperand(0);
@@ -232,39 +221,49 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::ADD8rr:    return MakeMRInst(X86::ADD8mr , FrameIndex, MI);
     case X86::ADD16rr:   return MakeMRInst(X86::ADD16mr, FrameIndex, MI);
     case X86::ADD32rr:   return MakeMRInst(X86::ADD32mr, FrameIndex, MI);
-    case X86::ADC32rr:   return MakeMRInst(X86::ADC32mr, FrameIndex, MI);
-    case X86::ADC32ri:   return MakeMIInst(X86::ADC32mi, FrameIndex, MI);
     case X86::ADD8ri:    return MakeMIInst(X86::ADD8mi , FrameIndex, MI);
     case X86::ADD16ri:   return MakeMIInst(X86::ADD16mi, FrameIndex, MI);
     case X86::ADD32ri:   return MakeMIInst(X86::ADD32mi, FrameIndex, MI);
+    case X86::ADD16ri8:  return MakeMIInst(X86::ADD16mi8,FrameIndex, MI);
+    case X86::ADD32ri8:  return MakeMIInst(X86::ADD32mi8,FrameIndex, MI);
+    case X86::ADC32rr:   return MakeMRInst(X86::ADC32mr, FrameIndex, MI);
+    case X86::ADC32ri:   return MakeMIInst(X86::ADC32mi, FrameIndex, MI);
+    case X86::ADC32ri8:  return MakeMIInst(X86::ADC32mi8,FrameIndex, MI);
     case X86::SUB8rr:    return MakeMRInst(X86::SUB8mr , FrameIndex, MI);
     case X86::SUB16rr:   return MakeMRInst(X86::SUB16mr, FrameIndex, MI);
     case X86::SUB32rr:   return MakeMRInst(X86::SUB32mr, FrameIndex, MI);
-    case X86::SBB32rr:   return MakeMRInst(X86::SBB32mr, FrameIndex, MI);
-    case X86::SBB8ri:    return MakeMIInst(X86::SBB8mi,  FrameIndex, MI);
-    case X86::SBB16ri:   return MakeMIInst(X86::SBB16mi, FrameIndex, MI);
-    case X86::SBB32ri:   return MakeMIInst(X86::SBB32mi, FrameIndex, MI);
     case X86::SUB8ri:    return MakeMIInst(X86::SUB8mi , FrameIndex, MI);
     case X86::SUB16ri:   return MakeMIInst(X86::SUB16mi, FrameIndex, MI);
     case X86::SUB32ri:   return MakeMIInst(X86::SUB32mi, FrameIndex, MI);
+    case X86::SUB16ri8:  return MakeMIInst(X86::SUB16mi8,FrameIndex, MI);
+    case X86::SUB32ri8:  return MakeMIInst(X86::SUB32mi8,FrameIndex, MI);
+    case X86::SBB32rr:   return MakeMRInst(X86::SBB32mr, FrameIndex, MI);
+    case X86::SBB32ri:   return MakeMIInst(X86::SBB32mi, FrameIndex, MI);
+    case X86::SBB32ri8:  return MakeMIInst(X86::SBB32mi8,FrameIndex, MI);
     case X86::AND8rr:    return MakeMRInst(X86::AND8mr , FrameIndex, MI);
     case X86::AND16rr:   return MakeMRInst(X86::AND16mr, FrameIndex, MI);
     case X86::AND32rr:   return MakeMRInst(X86::AND32mr, FrameIndex, MI);
     case X86::AND8ri:    return MakeMIInst(X86::AND8mi , FrameIndex, MI);
     case X86::AND16ri:   return MakeMIInst(X86::AND16mi, FrameIndex, MI);
     case X86::AND32ri:   return MakeMIInst(X86::AND32mi, FrameIndex, MI);
+    case X86::AND16ri8:  return MakeMIInst(X86::AND16mi8,FrameIndex, MI);
+    case X86::AND32ri8:  return MakeMIInst(X86::AND32mi8,FrameIndex, MI);
     case X86::OR8rr:     return MakeMRInst(X86::OR8mr ,  FrameIndex, MI);
     case X86::OR16rr:    return MakeMRInst(X86::OR16mr,  FrameIndex, MI);
     case X86::OR32rr:    return MakeMRInst(X86::OR32mr,  FrameIndex, MI);
     case X86::OR8ri:     return MakeMIInst(X86::OR8mi ,  FrameIndex, MI);
     case X86::OR16ri:    return MakeMIInst(X86::OR16mi,  FrameIndex, MI);
     case X86::OR32ri:    return MakeMIInst(X86::OR32mi,  FrameIndex, MI);
+    case X86::OR16ri8:   return MakeMIInst(X86::OR16mi8, FrameIndex, MI);
+    case X86::OR32ri8:   return MakeMIInst(X86::OR32mi8, FrameIndex, MI);
     case X86::XOR8rr:    return MakeMRInst(X86::XOR8mr , FrameIndex, MI);
     case X86::XOR16rr:   return MakeMRInst(X86::XOR16mr, FrameIndex, MI);
     case X86::XOR32rr:   return MakeMRInst(X86::XOR32mr, FrameIndex, MI);
     case X86::XOR8ri:    return MakeMIInst(X86::XOR8mi , FrameIndex, MI);
     case X86::XOR16ri:   return MakeMIInst(X86::XOR16mi, FrameIndex, MI);
     case X86::XOR32ri:   return MakeMIInst(X86::XOR32mi, FrameIndex, MI);
+    case X86::XOR16ri8:  return MakeMIInst(X86::XOR16mi8,FrameIndex, MI);
+    case X86::XOR32ri8:  return MakeMIInst(X86::XOR32mi8,FrameIndex, MI);
     case X86::SHL8rCL:   return MakeMInst( X86::SHL8mCL ,FrameIndex, MI);
     case X86::SHL16rCL:  return MakeMInst( X86::SHL16mCL,FrameIndex, MI);
     case X86::SHL32rCL:  return MakeMInst( X86::SHL32mCL,FrameIndex, MI);
@@ -329,6 +328,23 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::CMP8ri:    return MakeMIInst(X86::CMP8mi , FrameIndex, MI);
     case X86::CMP16ri:   return MakeMIInst(X86::CMP16mi, FrameIndex, MI);
     case X86::CMP32ri:   return MakeMIInst(X86::CMP32mi, FrameIndex, MI);
+    // Alias instructions
+    case X86::MOV8r0:    return MakeM0Inst(X86::MOV8mi, FrameIndex, MI);
+    case X86::MOV16r0:   return MakeM0Inst(X86::MOV16mi, FrameIndex, MI);
+    case X86::MOV32r0:   return MakeM0Inst(X86::MOV32mi, FrameIndex, MI);
+    // Alias scalar SSE instructions
+    case X86::FsMOVAPSrr: return MakeMRInst(X86::MOVSSmr, FrameIndex, MI);
+    case X86::FsMOVAPDrr: return MakeMRInst(X86::MOVSDmr, FrameIndex, MI);
+    // Scalar SSE instructions
+    case X86::MOVSSrr:   return MakeMRInst(X86::MOVSSmr, FrameIndex, MI);
+    case X86::MOVSDrr:   return MakeMRInst(X86::MOVSDmr, FrameIndex, MI);
+#if 0
+    // Packed SSE instructions
+    // FIXME: Can't use these until we are spilling XMM registers to
+    // 128-bit locations.
+    case X86::MOVAPSrr:  return MakeMRInst(X86::MOVAPSmr, FrameIndex, MI);
+    case X86::MOVAPDrr:  return MakeMRInst(X86::MOVAPDmr, FrameIndex, MI);
+#endif
     }
   } else if (i == 1) {
     switch(MI->getOpcode()) {
@@ -390,6 +406,8 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::IMUL32rr:  return MakeRMInst(X86::IMUL32rm,FrameIndex, MI);
     case X86::IMUL16rri: return MakeRMIInst(X86::IMUL16rmi, FrameIndex, MI);
     case X86::IMUL32rri: return MakeRMIInst(X86::IMUL32rmi, FrameIndex, MI);
+    case X86::IMUL16rri8:return MakeRMIInst(X86::IMUL16rmi8, FrameIndex, MI);
+    case X86::IMUL32rri8:return MakeRMIInst(X86::IMUL32rmi8, FrameIndex, MI);
     case X86::CMP8rr:    return MakeRMInst(X86::CMP8rm , FrameIndex, MI);
     case X86::CMP16rr:   return MakeRMInst(X86::CMP16rm, FrameIndex, MI);
     case X86::CMP32rr:   return MakeRMInst(X86::CMP32rm, FrameIndex, MI);
@@ -397,8 +415,49 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::MOVSX32rr8:return MakeRMInst(X86::MOVSX32rm8, FrameIndex, MI);
     case X86::MOVSX32rr16:return MakeRMInst(X86::MOVSX32rm16, FrameIndex, MI);
     case X86::MOVZX16rr8:return MakeRMInst(X86::MOVZX16rm8 , FrameIndex, MI);
-    case X86::MOVZX32rr8:  return MakeRMInst(X86::MOVZX32rm8, FrameIndex, MI);
+    case X86::MOVZX32rr8:return MakeRMInst(X86::MOVZX32rm8, FrameIndex, MI);
     case X86::MOVZX32rr16:return MakeRMInst(X86::MOVZX32rm16, FrameIndex, MI);
+    // Alias scalar SSE instructions
+    case X86::FsMOVAPSrr:return MakeRMInst(X86::MOVSSrm, FrameIndex, MI);
+    case X86::FsMOVAPDrr:return MakeRMInst(X86::MOVSDrm, FrameIndex, MI);
+    // Scalar SSE instructions
+    case X86::MOVSSrr:   return MakeRMInst(X86::MOVSSrm, FrameIndex, MI);
+    case X86::MOVSDrr:   return MakeRMInst(X86::MOVSDrm, FrameIndex, MI);
+    case X86::CVTTSS2SIrr:return MakeRMInst(X86::CVTTSS2SIrm, FrameIndex, MI);
+    case X86::CVTTSD2SIrr:return MakeRMInst(X86::CVTTSD2SIrm, FrameIndex, MI);
+    case X86::CVTSS2SDrr:return MakeRMInst(X86::CVTSS2SDrm, FrameIndex, MI);
+    case X86::CVTSD2SSrr:return MakeRMInst(X86::CVTSD2SSrm, FrameIndex, MI);
+    case X86::CVTSI2SSrr:return MakeRMInst(X86::CVTSI2SSrm, FrameIndex, MI);
+    case X86::CVTSI2SDrr:return MakeRMInst(X86::CVTSI2SDrm, FrameIndex, MI);
+    case X86::SQRTSSr:  return MakeRMInst(X86::SQRTSSm, FrameIndex, MI);
+    case X86::SQRTSDr:  return MakeRMInst(X86::SQRTSDm, FrameIndex, MI);
+    case X86::UCOMISSrr: return MakeRMInst(X86::UCOMISSrm, FrameIndex, MI);
+    case X86::UCOMISDrr: return MakeRMInst(X86::UCOMISDrm, FrameIndex, MI);
+    case X86::ADDSSrr:   return MakeRMInst(X86::ADDSSrm, FrameIndex, MI);
+    case X86::ADDSDrr:   return MakeRMInst(X86::ADDSDrm, FrameIndex, MI);
+    case X86::MULSSrr:   return MakeRMInst(X86::MULSSrm, FrameIndex, MI);
+    case X86::MULSDrr:   return MakeRMInst(X86::MULSDrm, FrameIndex, MI);
+    case X86::DIVSSrr:   return MakeRMInst(X86::DIVSSrm, FrameIndex, MI);
+    case X86::DIVSDrr:   return MakeRMInst(X86::DIVSDrm, FrameIndex, MI);
+    case X86::SUBSSrr:   return MakeRMInst(X86::SUBSSrm, FrameIndex, MI);
+    case X86::SUBSDrr:   return MakeRMInst(X86::SUBSDrm, FrameIndex, MI);
+    case X86::CMPSSrr:   return MakeRMInst(X86::CMPSSrm, FrameIndex, MI);
+    case X86::CMPSDrr:   return MakeRMInst(X86::CMPSDrm, FrameIndex, MI);
+#if 0
+    // Packed SSE instructions
+    // FIXME: Can't use these until we are spilling XMM registers to
+    // 128-bit locations.
+    case X86::ANDPSrr:   return MakeRMInst(X86::ANDPSrm, FrameIndex, MI);
+    case X86::ANDPDrr:   return MakeRMInst(X86::ANDPDrm, FrameIndex, MI);
+    case X86::ORPSrr:    return MakeRMInst(X86::ORPSrm, FrameIndex, MI);
+    case X86::ORPDrr:    return MakeRMInst(X86::ORPDrm, FrameIndex, MI);
+    case X86::XORPSrr:   return MakeRMInst(X86::XORPSrm, FrameIndex, MI);
+    case X86::XORPDrr:   return MakeRMInst(X86::XORPDrm, FrameIndex, MI);
+    case X86::ANDNPSrr:  return MakeRMInst(X86::ANDNPSrm, FrameIndex, MI);
+    case X86::ANDNPDrr:  return MakeRMInst(X86::ANDNPDrm, FrameIndex, MI);
+    case X86::MOVAPSrr:  return MakeRMInst(X86::MOVAPSrm, FrameIndex, MI);
+    case X86::MOVAPDrr:  return MakeRMInst(X86::MOVAPDrm, FrameIndex, MI);
+#endif
     }
   }
   if (PrintFailedFusing)
@@ -443,9 +502,11 @@ eliminateCallFramePseudoInstr(MachineFunction &MF, MachineBasicBlock &MBB,
         // factor out the amount the callee already popped.
         unsigned CalleeAmt = Old->getOperand(1).getImmedValue();
         Amount -= CalleeAmt;
-        if (Amount)
-          New = BuildMI(X86::ADD32ri, 1, X86::ESP,
+        if (Amount) {
+          unsigned Opc = Amount < 128 ? X86::ADD32ri8 : X86::ADD32ri;
+          New = BuildMI(Opc, 1, X86::ESP,
                         MachineOperand::UseAndDef).addZImm(Amount);
+        }
       }
 
       // Replace the pseudo instruction with a new instruction...
@@ -456,8 +517,9 @@ eliminateCallFramePseudoInstr(MachineFunction &MF, MachineBasicBlock &MBB,
     // something off the stack pointer, add it back.  We do this until we have
     // more advanced stack pointer tracking ability.
     if (unsigned CalleeAmt = I->getOperand(1).getImmedValue()) {
+      unsigned Opc = CalleeAmt < 128 ? X86::SUB32ri8 : X86::SUB32ri;
       MachineInstr *New =
-        BuildMI(X86::SUB32ri, 1, X86::ESP,
+        BuildMI(Opc, 1, X86::ESP,
                 MachineOperand::UseAndDef).addZImm(CalleeAmt);
       MBB.insert(I, New);
     }
@@ -517,8 +579,8 @@ void X86RegisterInfo::emitPrologue(MachineFunction &MF) const {
     int EBPOffset = MFI->getObjectOffset(MFI->getObjectIndexBegin())+4;
 
     if (NumBytes) {   // adjust stack pointer: ESP -= numbytes
-      MI= BuildMI(X86::SUB32ri, 1, X86::ESP, MachineOperand::UseAndDef)
-            .addZImm(NumBytes);
+      unsigned Opc = NumBytes < 128 ? X86::SUB32ri8 : X86::SUB32ri;
+      MI = BuildMI(Opc, 1, X86::ESP,MachineOperand::UseAndDef).addImm(NumBytes);
       MBB.insert(MBBI, MI);
     }
 
@@ -554,8 +616,8 @@ void X86RegisterInfo::emitPrologue(MachineFunction &MF) const {
 
     if (NumBytes) {
       // adjust stack pointer: ESP -= numbytes
-      MI= BuildMI(X86::SUB32ri, 1, X86::ESP, MachineOperand::UseAndDef)
-            .addZImm(NumBytes);
+      unsigned Opc = NumBytes < 128 ? X86::SUB32ri8 : X86::SUB32ri;
+      MI= BuildMI(Opc, 1, X86::ESP, MachineOperand::UseAndDef).addImm(NumBytes);
       MBB.insert(MBBI, MI);
     }
   }
@@ -595,11 +657,13 @@ void X86RegisterInfo::emitEpilogue(MachineFunction &MF,
       // instruction, merge the two instructions.
       if (MBBI != MBB.begin()) {
         MachineBasicBlock::iterator PI = prior(MBBI);
-        if (PI->getOpcode() == X86::ADD32ri &&
+        if ((PI->getOpcode() == X86::ADD32ri || 
+             PI->getOpcode() == X86::ADD32ri8) &&
             PI->getOperand(0).getReg() == X86::ESP) {
           NumBytes += PI->getOperand(1).getImmedValue();
           MBB.erase(PI);
-        } else if (PI->getOpcode() == X86::SUB32ri &&
+        } else if ((PI->getOpcode() == X86::SUB32ri ||
+                    PI->getOpcode() == X86::SUB32ri8) &&
                    PI->getOperand(0).getReg() == X86::ESP) {
           NumBytes -= PI->getOperand(1).getImmedValue();
           MBB.erase(PI);
@@ -609,15 +673,26 @@ void X86RegisterInfo::emitEpilogue(MachineFunction &MF,
         }
       }
 
-      if (NumBytes > 0)
-        BuildMI(MBB, MBBI, X86::ADD32ri, 2)
+      if (NumBytes > 0) {
+        unsigned Opc = NumBytes < 128 ? X86::ADD32ri8 : X86::ADD32ri;
+        BuildMI(MBB, MBBI, Opc, 2)
           .addReg(X86::ESP, MachineOperand::UseAndDef).addZImm(NumBytes);
-      else if ((int)NumBytes < 0)
-        BuildMI(MBB, MBBI, X86::SUB32ri, 2)
+      } else if ((int)NumBytes < 0) {
+        unsigned Opc = -NumBytes < 128 ? X86::SUB32ri8 : X86::SUB32ri;
+        BuildMI(MBB, MBBI, Opc, 2)
           .addReg(X86::ESP, MachineOperand::UseAndDef).addZImm(-NumBytes);
+      }
     }
   }
 }
 
+unsigned X86RegisterInfo::getRARegister() const {
+  return X86::ST0;  // use a non-register register
+}
+
+unsigned X86RegisterInfo::getFrameRegister(MachineFunction &MF) const {
+  return hasFP(MF) ? X86::EBP : X86::ESP;
+}
+
 #include "X86GenRegisterInfo.inc"
 
