@@ -22,15 +22,17 @@
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"
 #include "llvm/ADT/Statistic.h"
+#include <iostream>
 #include <set>
 using namespace llvm;
 
 namespace {
   Statistic<> NumInlined("inline", "Number of functions inlined");
-  Statistic<> NumDeleted("inline", "Number of functions deleted because all callers found");
+  Statistic<> NumDeleted("inline",
+                       "Number of functions deleted because all callers found");
   cl::opt<unsigned>             // FIXME: 200 is VERY conservative
   InlineLimit("inline-threshold", cl::Hidden, cl::init(200),
-              cl::desc("Control the amount of inlining to perform (default = 200)"));
+        cl::desc("Control the amount of inlining to perform (default = 200)"));
 }
 
 Inliner::Inliner() : InlineThreshold(InlineLimit) {}
@@ -41,18 +43,7 @@ static bool InlineCallIfPossible(CallSite CS, CallGraph &CG,
                                  const std::set<Function*> &SCCFunctions) {
   Function *Caller = CS.getInstruction()->getParent()->getParent();
   Function *Callee = CS.getCalledFunction();
-  if (!InlineFunction(CS)) return false;
-
-  // Update the call graph by deleting the edge from Callee to Caller
-  CallGraphNode *CalleeNode = CG[Callee];
-  CallGraphNode *CallerNode = CG[Caller];
-  CallerNode->removeCallEdgeTo(CalleeNode);
-
-  // Since we inlined all uninlined call sites in the callee into the caller,
-  // add edges from the caller to all of the callees of the callee.
-  for (CallGraphNode::iterator I = CalleeNode->begin(),
-         E = CalleeNode->end(); I != E; ++I)
-    CallerNode->addCalledFunction(*I);
+  if (!InlineFunction(CS, &CG)) return false;
 
   // If we inlined the last possible call site to the function, delete the
   // function body now.
@@ -62,6 +53,7 @@ static bool InlineCallIfPossible(CallSite CS, CallGraph &CG,
                     << Callee->getName() << "\n");
 
     // Remove any call graph edges from the callee to its callees.
+    CallGraphNode *CalleeNode = CG[Callee];
     while (CalleeNode->begin() != CalleeNode->end())
       CalleeNode->removeCallEdgeTo(*(CalleeNode->end()-1));
 
