@@ -57,6 +57,7 @@ namespace {
     // AA - If we have an alias analysis object to update, this is it, otherwise
     // this is null.
     AliasAnalysis *AA;
+    LoopInfo *LI;
 
     virtual bool runOnFunction(Function &F);
 
@@ -69,6 +70,7 @@ namespace {
       AU.addPreserved<LoopInfo>();
       AU.addPreserved<DominatorSet>();
       AU.addPreserved<ImmediateDominators>();
+      AU.addPreserved<ETForest>();
       AU.addPreserved<DominatorTree>();
       AU.addPreserved<DominanceFrontier>();
       AU.addPreservedID(BreakCriticalEdgesID);  // No critical edges added.
@@ -99,22 +101,25 @@ FunctionPass *llvm::createLoopSimplifyPass() { return new LoopSimplify(); }
 ///
 bool LoopSimplify::runOnFunction(Function &F) {
   bool Changed = false;
-  LoopInfo &LI = getAnalysis<LoopInfo>();
+  LI = &getAnalysis<LoopInfo>();
   AA = getAnalysisToUpdate<AliasAnalysis>();
 
-  for (LoopInfo::iterator I = LI.begin(), E = LI.end(); I != E; ++I)
+  for (LoopInfo::iterator I = LI->begin(), E = LI->end(); I != E; ++I)
     Changed |= ProcessLoop(*I);
 
   return Changed;
 }
 
-
 /// ProcessLoop - Walk the loop structure in depth first order, ensuring that
 /// all loops have preheaders.
 ///
 bool LoopSimplify::ProcessLoop(Loop *L) {
   bool Changed = false;
-
+  // Canonicalize inner loops before outer loops.  Inner loop canonicalization
+  // can provide work for the outer loop to canonicalize.
+  for (Loop::iterator I = L->begin(), E = L->end(); I != E; ++I)
+    Changed |= ProcessLoop(*I);
+  
   // Check to see that no blocks (other than the header) in the loop have
   // predecessors that are not in the loop.  This is not valid for natural
   // loops, but can occur if the blocks are unreachable.  Since they are
@@ -159,13 +164,15 @@ bool LoopSimplify::ProcessLoop(Loop *L) {
   // predecessors from outside of the loop, split the edge now.
   std::vector<BasicBlock*> ExitBlocks;
   L->getExitBlocks(ExitBlocks);
-
+    
   SetVector<BasicBlock*> ExitBlockSet(ExitBlocks.begin(), ExitBlocks.end());
   for (SetVector<BasicBlock*>::iterator I = ExitBlockSet.begin(),
          E = ExitBlockSet.end(); I != E; ++I) {
     BasicBlock *ExitBlock = *I;
     for (pred_iterator PI = pred_begin(ExitBlock), PE = pred_end(ExitBlock);
          PI != PE; ++PI)
+      // Must be exactly this loop: no subloops, parent loops, or non-loop preds
+      // allowed.
       if (!L->contains(*PI)) {
         RewriteLoopExitBlock(L, ExitBlock);
         NumInserted++;
@@ -177,6 +184,7 @@ bool LoopSimplify::ProcessLoop(Loop *L) {
   // If the header has more than two predecessors at this point (from the
   // preheader and from multiple backedges), we must adjust the loop.
   if (L->getNumBackEdges() != 1) {
+    
     // If this is really a nested loop, rip it out into a child loop.
     if (Loop *NL = SeparateNestedLoop(L)) {
       ++NumNested;
@@ -194,7 +202,6 @@ bool LoopSimplify::ProcessLoop(Loop *L) {
   // incoming values (the loop is canonicalized), we may have simplified the PHI
   // down to 'X = phi [X, Y]', which should be replaced with 'Y'.
   PHINode *PN;
-  DominatorSet &DS = getAnalysis<DominatorSet>();
   for (BasicBlock::iterator I = L->getHeader()->begin();
        (PN = dyn_cast<PHINode>(I++)); )
     if (Value *V = PN->hasConstantValue()) {
@@ -202,9 +209,6 @@ bool LoopSimplify::ProcessLoop(Loop *L) {
         PN->eraseFromParent();
       }
 
-  for (Loop::iterator I = L->begin(), E = L->end(); I != E; ++I)
-    Changed |= ProcessLoop(*I);
-
   return Changed;
 }
 
@@ -310,8 +314,8 @@ void LoopSimplify::InsertPreheaderForLoop(Loop *L) {
   std::vector<BasicBlock*> OutsideBlocks;
   for (pred_iterator PI = pred_begin(Header), PE = pred_end(Header);
        PI != PE; ++PI)
-      if (!L->contains(*PI))           // Coming in from outside the loop?
-        OutsideBlocks.push_back(*PI);  // Keep track of it...
+    if (!L->contains(*PI))           // Coming in from outside the loop?
+      OutsideBlocks.push_back(*PI);  // Keep track of it...
 
   // Split out the loop pre-header
   BasicBlock *NewBB =
@@ -323,7 +327,7 @@ void LoopSimplify::InsertPreheaderForLoop(Loop *L) {
 
   // We know that we have loop information to update... update it now.
   if (Loop *Parent = L->getParentLoop())
-    Parent->addBasicBlockToLoop(NewBB, getAnalysis<LoopInfo>());
+    Parent->addBasicBlockToLoop(NewBB, *LI);
 
   DominatorSet &DS = getAnalysis<DominatorSet>();  // Update dominator info
   DominatorTree &DT = getAnalysis<DominatorTree>();
@@ -334,6 +338,7 @@ void LoopSimplify::InsertPreheaderForLoop(Loop *L) {
   // the old header.
   DominatorTree::Node *PHDomTreeNode =
     DT.createNewNode(NewBB, DT.getNode(Header)->getIDom());
+  BasicBlock *oldHeaderIDom = DT.getNode(Header)->getIDom()->getBlock();
 
   // Change the header node so that PNHode is the new immediate dominator
   DT.changeImmediateDominator(DT.getNode(Header), PHDomTreeNode);
@@ -359,6 +364,15 @@ void LoopSimplify::InsertPreheaderForLoop(Loop *L) {
     // The preheader now is the immediate dominator for the header node...
     ID->setImmediateDominator(Header, NewBB);
   }
+  
+  // Update ET Forest information if we have it...
+  if (ETForest *EF = getAnalysisToUpdate<ETForest>()) {
+    // Whatever i-dominated the header node now immediately dominates NewBB
+    EF->addNewBlock(NewBB, oldHeaderIDom);
+
+    // The preheader now is the immediate dominator for the header node...
+    EF->setImmediateDominator(Header, NewBB);
+  }
 
   // Update dominance frontier information...
   if (DominanceFrontier *DF = getAnalysisToUpdate<DominanceFrontier>()) {
@@ -401,8 +415,6 @@ void LoopSimplify::InsertPreheaderForLoop(Loop *L) {
 /// blocks.  This method is used to split exit blocks that have predecessors
 /// outside of the loop.
 BasicBlock *LoopSimplify::RewriteLoopExitBlock(Loop *L, BasicBlock *Exit) {
-  DominatorSet &DS = getAnalysis<DominatorSet>();
-
   std::vector<BasicBlock*> LoopBlocks;
   for (pred_iterator I = pred_begin(Exit), E = pred_end(Exit); I != E; ++I)
     if (L->contains(*I))
@@ -411,10 +423,16 @@ BasicBlock *LoopSimplify::RewriteLoopExitBlock(Loop *L, BasicBlock *Exit) {
   assert(!LoopBlocks.empty() && "No edges coming in from outside the loop?");
   BasicBlock *NewBB = SplitBlockPredecessors(Exit, ".loopexit", LoopBlocks);
 
-  // Update Loop Information - we know that the new block will be in the parent
-  // loop of L.
-  if (Loop *Parent = L->getParentLoop())
-    Parent->addBasicBlockToLoop(NewBB, getAnalysis<LoopInfo>());
+  // Update Loop Information - we know that the new block will be in whichever
+  // loop the Exit block is in.  Note that it may not be in that immediate loop,
+  // if the successor is some other loop header.  In that case, we continue 
+  // walking up the loop tree to find a loop that contains both the successor
+  // block and the predecessor block.
+  Loop *SuccLoop = LI->getLoopFor(Exit);
+  while (SuccLoop && !SuccLoop->contains(L->getHeader()))
+    SuccLoop = SuccLoop->getParentLoop();
+  if (SuccLoop)
+    SuccLoop->addBasicBlockToLoop(NewBB, *LI);
 
   // Update dominator information (set, immdom, domtree, and domfrontier)
   UpdateDomInfoForRevectoredPreds(NewBB, LoopBlocks);
@@ -498,17 +516,15 @@ Loop *LoopSimplify::SeparateNestedLoop(Loop *L) {
   // Create the new outer loop.
   Loop *NewOuter = new Loop();
 
-  LoopInfo &LI = getAnalysis<LoopInfo>();
-
   // Change the parent loop to use the outer loop as its child now.
   if (Loop *Parent = L->getParentLoop())
     Parent->replaceChildLoopWith(L, NewOuter);
   else
-    LI.changeTopLevelLoop(L, NewOuter);
+    LI->changeTopLevelLoop(L, NewOuter);
 
   // This block is going to be our new header block: add it to this loop and all
   // parent loops.
-  NewOuter->addBasicBlockToLoop(NewBB, getAnalysis<LoopInfo>());
+  NewOuter->addBasicBlockToLoop(NewBB, *LI);
 
   // L is now a subloop of our outer loop.
   NewOuter->addChildLoop(L);
@@ -540,8 +556,8 @@ Loop *LoopSimplify::SeparateNestedLoop(Loop *L) {
     if (!BlocksInL.count(BB)) {
       // Move this block to the parent, updating the exit blocks sets
       L->removeBlockFromLoop(BB);
-      if (LI[BB] == L)
-        LI.changeLoopFor(BB, NewOuter);
+      if ((*LI)[BB] == L)
+        LI->changeLoopFor(BB, NewOuter);
       --i;
     }
   }
@@ -643,7 +659,7 @@ void LoopSimplify::InsertUniqueBackedgeBlock(Loop *L) {
 
   // Update Loop Information - we know that this block is now in the current
   // loop and all parent loops.
-  L->addBasicBlockToLoop(BEBlock, getAnalysis<LoopInfo>());
+  L->addBasicBlockToLoop(BEBlock, *LI);
 
   // Update dominator information (set, immdom, domtree, and domfrontier)
   UpdateDomInfoForRevectoredPreds(BEBlock, BackedgeBlocks);
@@ -762,6 +778,7 @@ void LoopSimplify::UpdateDomInfoForRevectoredPreds(BasicBlock *NewBB,
         NewBBIDomNode = NewBBIDomNode->getIDom();
         assert(NewBBIDomNode && "No shared dominator found??");
       }
+      NewBBIDom = NewBBIDomNode->getBlock();
     }
 
     // Create the new dominator tree node... and set the idom of NewBB.
@@ -775,6 +792,13 @@ void LoopSimplify::UpdateDomInfoForRevectoredPreds(BasicBlock *NewBB,
     }
   }
 
+  // Update ET-Forest information if it is active.
+  if (ETForest *EF = getAnalysisToUpdate<ETForest>()) {
+    EF->addNewBlock(NewBB, NewBBIDom);
+    if (NewBBDominatesNewBBSucc)
+      EF->setImmediateDominator(NewBBSucc, NewBB);
+  }
+
   // Update dominance frontier information...
   if (DominanceFrontier *DF = getAnalysisToUpdate<DominanceFrontier>()) {
     // If NewBB dominates NewBBSucc, then DF(NewBB) is now going to be the
