@@ -16,8 +16,6 @@
 #include "llvm/Constants.h"
 #include "llvm/GlobalValue.h"
 #include "llvm/Instruction.h"
-#include <iostream>
-
 using namespace llvm;
 
 Value *llvm::MapValue(const Value *V, std::map<const Value*, Value*> &VM) {
@@ -26,13 +24,13 @@ Value *llvm::MapValue(const Value *V, std::map<const Value*, Value*> &VM) {
 
   // Global values do not need to be seeded into the ValueMap if they are using
   // the identity mapping.
-  if (isa<GlobalValue>(V))
+  if (isa<GlobalValue>(V) || isa<InlineAsm>(V))
     return VMSlot = const_cast<Value*>(V);
 
   if (Constant *C = const_cast<Constant*>(dyn_cast<Constant>(V))) {
     if (isa<ConstantIntegral>(C) || isa<ConstantFP>(C) ||
         isa<ConstantPointerNull>(C) || isa<ConstantAggregateZero>(C) ||
-        isa<UndefValue>(C))
+        isa<UndefValue>(C) || isa<InlineAsm>(V))
       return VMSlot = C;           // Primitive constants map directly
     else if (ConstantArray *CA = dyn_cast<ConstantArray>(C)) {
       for (unsigned i = 0, e = CA->getNumOperands(); i != e; ++i) {
@@ -87,6 +85,20 @@ Value *llvm::MapValue(const Value *V, std::map<const Value*, Value*> &VM) {
         Constant *MV2 = cast<Constant>(MapValue(CE->getOperand(1), VM));
         Constant *MV3 = cast<Constant>(MapValue(CE->getOperand(2), VM));
         return VMSlot = ConstantExpr::getSelect(MV1, MV2, MV3);
+      } else if (CE->getOpcode() == Instruction::InsertElement) {
+        Constant *MV1 = cast<Constant>(MapValue(CE->getOperand(0), VM));
+        Constant *MV2 = cast<Constant>(MapValue(CE->getOperand(1), VM));
+        Constant *MV3 = cast<Constant>(MapValue(CE->getOperand(2), VM));
+        return VMSlot = ConstantExpr::getInsertElement(MV1, MV2, MV3);
+      } else if (CE->getOpcode() == Instruction::ExtractElement) {
+        Constant *MV1 = cast<Constant>(MapValue(CE->getOperand(0), VM));
+        Constant *MV2 = cast<Constant>(MapValue(CE->getOperand(1), VM));
+        return VMSlot = ConstantExpr::getExtractElement(MV1, MV2);
+      } else if (CE->getOpcode() == Instruction::ShuffleVector) {
+        Constant *MV1 = cast<Constant>(MapValue(CE->getOperand(0), VM));
+        Constant *MV2 = cast<Constant>(MapValue(CE->getOperand(1), VM));
+        Constant *MV3 = cast<Constant>(CE->getOperand(2));
+        return VMSlot = ConstantExpr::getShuffleVector(MV1, MV2, MV3);
       } else {
         assert(CE->getNumOperands() == 2 && "Must be binary operator?");
         Constant *MV1 = cast<Constant>(MapValue(CE->getOperand(0), VM));
@@ -94,6 +106,25 @@ Value *llvm::MapValue(const Value *V, std::map<const Value*, Value*> &VM) {
         return VMSlot = ConstantExpr::get(CE->getOpcode(), MV1, MV2);
       }
 
+    } else if (ConstantPacked *CP = dyn_cast<ConstantPacked>(C)) {
+      for (unsigned i = 0, e = CP->getNumOperands(); i != e; ++i) {
+        Value *MV = MapValue(CP->getOperand(i), VM);
+        if (MV != CP->getOperand(i)) {
+          // This packed value must contain a reference to a global, make a new
+          // packed constant and return it.
+          //
+          std::vector<Constant*> Values;
+          Values.reserve(CP->getNumOperands());
+          for (unsigned j = 0; j != i; ++j)
+            Values.push_back(CP->getOperand(j));
+          Values.push_back(cast<Constant>(MV));
+          for (++i; i != e; ++i)
+            Values.push_back(cast<Constant>(MapValue(CP->getOperand(i), VM)));
+          return VMSlot = ConstantPacked::get(Values);
+        }
+      }
+      return VMSlot = C;
+      
     } else {
       assert(0 && "Unknown type of constant!");
     }
@@ -112,12 +143,6 @@ void llvm::RemapInstruction(Instruction *I,
   for (unsigned op = 0, E = I->getNumOperands(); op != E; ++op) {
     const Value *Op = I->getOperand(op);
     Value *V = MapValue(Op, ValueMap);
-#ifndef NDEBUG
-    if (!V) {
-      std::cerr << "Val = \n" << *Op << "Addr = " << (void*)Op;
-      std::cerr << "\nInst = " << *I;
-    }
-#endif
     assert(V && "Referenced value not in value map!");
     I->setOperand(op, V);
   }
