@@ -123,10 +123,68 @@ public:
 
 static TimingInfo *TheTimeInfo;
 
-//===----------------------------------------------------------------------===//
-// Declare the PassManagerTraits which will be specialized...
-//
-template<class UnitType> class PassManagerTraits;   // Do not define.
+struct BBTraits {
+  typedef BasicBlock UnitType;
+  
+  // PassClass - The type of passes tracked by this PassManager
+  typedef BasicBlockPass PassClass;
+
+  // SubPassClass - The types of classes that should be collated together
+  // This is impossible to match, so BasicBlock instantiations of PassManagerT
+  // do not collate.
+  //
+  typedef BasicBlockPassManager SubPassClass;
+
+  // BatcherClass - The type to use for collation of subtypes... This class is
+  // never instantiated for the BasicBlockPassManager, but it must be an
+  // instance of PassClass to typecheck.
+  //
+  typedef PassClass BatcherClass;
+
+  // ParentClass - The type of the parent PassManager...
+  typedef FunctionPassManagerT ParentClass;
+
+  // PMType - The type of this passmanager
+  typedef BasicBlockPassManager PMType;
+};
+
+struct FTraits {
+  typedef Function UnitType;
+  
+  // PassClass - The type of passes tracked by this PassManager
+  typedef FunctionPass PassClass;
+
+  // SubPassClass - The types of classes that should be collated together
+  typedef BasicBlockPass SubPassClass;
+
+  // BatcherClass - The type to use for collation of subtypes...
+  typedef BasicBlockPassManager BatcherClass;
+
+  // ParentClass - The type of the parent PassManager...
+  typedef ModulePassManager ParentClass;
+
+  // PMType - The type of this passmanager
+  typedef FunctionPassManagerT PMType;
+};
+
+struct MTraits {
+  typedef Module UnitType;
+  
+  // PassClass - The type of passes tracked by this PassManager
+  typedef ModulePass PassClass;
+
+  // SubPassClass - The types of classes that should be collated together
+  typedef FunctionPass SubPassClass;
+
+  // BatcherClass - The type to use for collation of subtypes...
+  typedef FunctionPassManagerT BatcherClass;
+
+  // ParentClass - The type of the parent PassManager...
+  typedef AnalysisResolver ParentClass;
+  
+  // PMType - The type of this passmanager
+  typedef ModulePassManager PMType;
+};
 
 
 //===----------------------------------------------------------------------===//
@@ -134,25 +192,26 @@ template<class UnitType> class PassManagerTraits;   // Do not define.
 // deletes all passes contained inside of the PassManagerT, so you shouldn't
 // delete passes manually, and all passes should be dynamically allocated.
 //
-template<typename UnitType>
-class PassManagerT : public PassManagerTraits<UnitType>,public AnalysisResolver{
-  typedef PassManagerTraits<UnitType> Traits;
-  typedef typename Traits::PassClass       PassClass;
-  typedef typename Traits::SubPassClass SubPassClass;
-  typedef typename Traits::BatcherClass BatcherClass;
-  typedef typename Traits::ParentClass   ParentClass;
-
-#if defined(_MSC_VER) || defined(__INTEL_COMPILER) || defined(__HP_aCC)
-  friend PassClass;
-  friend SubPassClass;
-#else
-  friend class PassManagerTraits<UnitType>::PassClass;
-  friend class PassManagerTraits<UnitType>::SubPassClass;
-#endif
-  friend class PassManagerTraits<UnitType>;
+template<typename Trait> class PassManagerT : public AnalysisResolver {
+  
+  typedef typename Trait::PassClass    PassClass;
+  typedef typename Trait::UnitType     UnitType;
+  typedef typename Trait::ParentClass  ParentClass;
+  typedef typename Trait::SubPassClass SubPassClass;
+  typedef typename Trait::BatcherClass BatcherClass;
+  typedef typename Trait::PMType       PMType;
+  
+  friend class ModulePass;
+  friend class FunctionPass;
+  friend class BasicBlockPass;
+  
   friend class ImmutablePass;
+  
+  friend class BasicBlockPassManager;
+  friend class FunctionPassManagerT;
+  friend class ModulePassManager;
 
-  std::vector<PassClass*> Passes;    // List of passes to run
+  std::vector<PassClass*> Passes;               // List of passes to run
   std::vector<ImmutablePass*> ImmutablePasses;  // List of immutable passes
 
   // The parent of this pass manager...
@@ -174,8 +233,20 @@ class PassManagerT : public PassManagerTraits<UnitType>,public AnalysisResolver{
   std::map<Pass*, Pass*> LastUseOf;
 
 public:
+  
+  // getPMName() - Return the name of the unit the PassManager operates on for
+  // debugging.
+  virtual const char *getPMName() const =0;
+  
+  virtual const char *getPassName() const =0;
+
+  virtual bool runPass(PassClass *P, UnitType *M) =0;
+  
+  // TODO:Figure out what pure virtuals remain.
+  
+  
   PassManagerT(ParentClass *Par = 0) : Parent(Par), Batcher(0) {}
-  ~PassManagerT() {
+  virtual ~PassManagerT() {
     // Delete all of the contained passes...
     for (typename std::vector<PassClass*>::iterator
            I = Passes.begin(), E = Passes.end(); I != E; ++I)
@@ -189,23 +260,12 @@ public:
   // run - Run all of the queued passes on the specified module in an optimal
   // way.
   virtual bool runOnUnit(UnitType *M) {
-    bool MadeChanges = false;
     closeBatcher();
     CurrentAnalyses.clear();
 
     TimingInfo::createTheTimeInfo();
 
-    // Add any immutable passes to the CurrentAnalyses set...
-    for (unsigned i = 0, e = ImmutablePasses.size(); i != e; ++i) {
-      ImmutablePass *IPass = ImmutablePasses[i];
-      if (const PassInfo *PI = IPass->getPassInfo()) {
-        CurrentAnalyses[PI] = IPass;
-
-        const std::vector<const PassInfo*> &II = PI->getInterfacesImplemented();
-        for (unsigned i = 0, e = II.size(); i != e; ++i)
-          CurrentAnalyses[II[i]] = IPass;
-      }
-    }
+    addImmutablePasses();
 
     // LastUserOf - This contains the inverted LastUseOfMap...
     std::map<Pass *, std::vector<Pass*> > LastUserOf;
@@ -214,123 +274,22 @@ public:
       LastUserOf[I->second].push_back(I->first);
 
     // Output debug information...
-    if (Parent == 0) PMDebug::PerformPassStartupStuff(this);
+    assert(dynamic_cast<PassClass*>(this) && 
+           "It wasn't the PassClass I thought it was");
+    if (Parent == 0) 
+      PMDebug::PerformPassStartupStuff((dynamic_cast<PMType*>(this)));
 
-    // Run all of the passes
-    for (unsigned i = 0, e = Passes.size(); i < e; ++i) {
-      PassClass *P = Passes[i];
-
-      PMDebug::PrintPassInformation(getDepth(), "Executing Pass", P, M);
-
-      // Get information about what analyses the pass uses...
-      AnalysisUsage AnUsage;
-      P->getAnalysisUsage(AnUsage);
-      PMDebug::PrintAnalysisSetInfo(getDepth(), "Required", P,
-                                    AnUsage.getRequiredSet());
-
-      // All Required analyses should be available to the pass as it runs!  Here
-      // we fill in the AnalysisImpls member of the pass so that it can
-      // successfully use the getAnalysis() method to retrieve the
-      // implementations it needs.
-      //
-      P->AnalysisImpls.clear();
-      P->AnalysisImpls.reserve(AnUsage.getRequiredSet().size());
-      for (std::vector<const PassInfo *>::const_iterator
-             I = AnUsage.getRequiredSet().begin(),
-             E = AnUsage.getRequiredSet().end(); I != E; ++I) {
-        Pass *Impl = getAnalysisOrNullUp(*I);
-        if (Impl == 0) {
-          std::cerr << "Analysis '" << (*I)->getPassName()
-                    << "' used but not available!";
-          assert(0 && "Analysis used but not available!");
-        } else if (PassDebugging == Details) {
-          if ((*I)->getPassName() != std::string(Impl->getPassName()))
-            std::cerr << "    Interface '" << (*I)->getPassName()
-                    << "' implemented by '" << Impl->getPassName() << "'\n";
-        }
-        P->AnalysisImpls.push_back(std::make_pair(*I, Impl));
-      }
-
-      // Run the sub pass!
-      if (TheTimeInfo) TheTimeInfo->passStarted(P);
-      bool Changed = runPass(P, M);
-      if (TheTimeInfo) TheTimeInfo->passEnded(P);
-      MadeChanges |= Changed;
-
-      // Check for memory leaks by the pass...
-      LeakDetector::checkForGarbage(std::string("after running pass '") +
-                                    P->getPassName() + "'");
-
-      if (Changed)
-        PMDebug::PrintPassInformation(getDepth()+1, "Made Modification", P, M);
-      PMDebug::PrintAnalysisSetInfo(getDepth(), "Preserved", P,
-                                    AnUsage.getPreservedSet());
-
-
-      // Erase all analyses not in the preserved set...
-      if (!AnUsage.getPreservesAll()) {
-        const std::vector<AnalysisID> &PreservedSet = AnUsage.getPreservedSet();
-        for (std::map<AnalysisID, Pass*>::iterator I = CurrentAnalyses.begin(),
-               E = CurrentAnalyses.end(); I != E; )
-          if (std::find(PreservedSet.begin(), PreservedSet.end(), I->first) !=
-              PreservedSet.end())
-            ++I; // This analysis is preserved, leave it in the available set...
-          else {
-            if (!dynamic_cast<ImmutablePass*>(I->second)) {
-              std::map<AnalysisID, Pass*>::iterator J = I++;
-              CurrentAnalyses.erase(J);   // Analysis not preserved!
-            } else {
-              ++I;
-            }
-          }
-      }
-
-      // Add the current pass to the set of passes that have been run, and are
-      // thus available to users.
-      //
-      if (const PassInfo *PI = P->getPassInfo()) {
-        CurrentAnalyses[PI] = P;
-
-        // This pass is the current implementation of all of the interfaces it
-        // implements as well.
-        //
-        const std::vector<const PassInfo*> &II = PI->getInterfacesImplemented();
-        for (unsigned i = 0, e = II.size(); i != e; ++i)
-          CurrentAnalyses[II[i]] = P;
-      }
-
-      // Free memory for any passes that we are the last use of...
-      std::vector<Pass*> &DeadPass = LastUserOf[P];
-      for (std::vector<Pass*>::iterator I = DeadPass.begin(),E = DeadPass.end();
-           I != E; ++I) {
-        PMDebug::PrintPassInformation(getDepth()+1, "Freeing Pass", *I, M);
-        (*I)->releaseMemory();
-      }
-
-      // Make sure to remove dead passes from the CurrentAnalyses list...
-      for (std::map<AnalysisID, Pass*>::iterator I = CurrentAnalyses.begin();
-           I != CurrentAnalyses.end(); ) {
-        std::vector<Pass*>::iterator DPI = std::find(DeadPass.begin(),
-                                                     DeadPass.end(), I->second);
-        if (DPI != DeadPass.end()) {    // This pass is dead now... remove it
-          std::map<AnalysisID, Pass*>::iterator IDead = I++;
-          CurrentAnalyses.erase(IDead);
-        } else {
-          ++I;  // Move on to the next element...
-        }
-      }
-    }
-
-    return MadeChanges;
+    return runPasses(M, LastUserOf);
   }
 
   // dumpPassStructure - Implement the -debug-passes=PassStructure option
-  virtual void dumpPassStructure(unsigned Offset = 0) {
+  inline void dumpPassStructure(unsigned Offset = 0) {
     // Print out the immutable passes...
+    
     for (unsigned i = 0, e = ImmutablePasses.size(); i != e; ++i)
       ImmutablePasses[i]->dumpPassStructure(0);
 
-    std::cerr << std::string(Offset*2, ' ') << Traits::getPMName()
+    std::cerr << std::string(Offset*2, ' ') << this->getPMName()
               << " Pass Manager\n";
     for (typename std::vector<PassClass*>::iterator
            I = Passes.begin(), E = Passes.end(); I != E; ++I) {
@@ -418,7 +377,9 @@ public:
       // frees the analysis AFTER this pass manager runs.
       //
       if (Parent) {
-        Parent->markPassUsed(P, this);
+        assert(dynamic_cast<Pass*>(this) && 
+               "It wasn't the Pass type I thought it was.");
+        Parent->markPassUsed(P, dynamic_cast<Pass*>(this));
       } else {
         assert(getAnalysisOrNullUp(P) &&
                dynamic_cast<ImmutablePass*>(getAnalysisOrNullUp(P)) &&
@@ -435,6 +396,7 @@ public:
   }
 
   virtual unsigned getNumContainedPasses() const { return Passes.size(); }
+  
   virtual const Pass *getContainedPass(unsigned N) const {
     assert(N < Passes.size() && "Pass number out of range!");
     return Passes[N];
@@ -449,24 +411,16 @@ public:
     // Get information about what analyses the pass uses...
     AnalysisUsage AnUsage;
     P->getAnalysisUsage(AnUsage);
-    const std::vector<AnalysisID> &Required = AnUsage.getRequiredSet();
-
-    // Loop over all of the analyses used by this pass,
-    for (std::vector<AnalysisID>::const_iterator I = Required.begin(),
-           E = Required.end(); I != E; ++I) {
-      if (getAnalysisOrNullDown(*I) == 0) {
-        Pass *AP = (*I)->createPass();
-        if (ImmutablePass *IP = dynamic_cast<ImmutablePass *> (AP)) { add(IP); }
-        else if (PassClass *RP = dynamic_cast<PassClass *> (AP)) { add(RP); }
-        else { assert (0 && "Wrong kind of pass for this PassManager"); }
-      }
-    }
-
+    
+    addRequiredPasses(AnUsage.getRequiredSet());
+    
     // Tell the pass to add itself to this PassManager... the way it does so
     // depends on the class of the pass, and is critical to laying out passes in
     // an optimal order..
     //
-    P->addToPassManager(this, AnUsage);
+    assert(dynamic_cast<PMType*>(this) && 
+        "It wasn't the right passmanager type.");
+    P->addToPassManager(static_cast<PMType*>(this), AnUsage);
   }
 
   // add - H4x0r an ImmutablePass into a PassManager that might not be
@@ -476,19 +430,9 @@ public:
     // Get information about what analyses the pass uses...
     AnalysisUsage AnUsage;
     P->getAnalysisUsage(AnUsage);
-    const std::vector<AnalysisID> &Required = AnUsage.getRequiredSet();
-
-    // Loop over all of the analyses used by this pass,
-    for (std::vector<AnalysisID>::const_iterator I = Required.begin(),
-           E = Required.end(); I != E; ++I) {
-      if (getAnalysisOrNullDown(*I) == 0) {
-        Pass *AP = (*I)->createPass();
-        if (ImmutablePass *IP = dynamic_cast<ImmutablePass *> (AP)) add(IP);
-        else if (PassClass *RP = dynamic_cast<PassClass *> (AP)) add(RP);
-        else assert (0 && "Wrong kind of pass for this PassManager");
-      }
-    }
-
+    
+    addRequiredPasses(AnUsage.getRequiredSet());
+    
     // Add the ImmutablePass to this PassManager.
     addPass(P, AnUsage);
   }
@@ -508,11 +452,10 @@ private:
     const std::vector<AnalysisID> &RequiredSet = AnUsage.getRequiredSet();
 
     // FIXME: If this pass being added isn't killed by any of the passes in the
-    // batcher class then we can reorder to pass to execute before the batcher
+    // batcher class then we can reorder the pass to execute before the batcher
     // does, which will potentially allow us to batch more passes!
     //
-    //const std::vector<AnalysisID> &ProvidedSet = AnUsage.getProvidedSet();
-    if (Batcher /*&& ProvidedSet.empty()*/)
+    if (Batcher)
       closeBatcher();                     // This pass cannot be batched!
 
     // Set the Resolver instance variable in the Pass so that it knows where to
@@ -529,44 +472,26 @@ private:
            E = RequiredSet.end(); I != E; ++I)
       markPassUsed(*I, P);     // Mark *I as used by P
 
-    // Erase all analyses not in the preserved set...
-    if (!AnUsage.getPreservesAll()) {
-      const std::vector<AnalysisID> &PreservedSet = AnUsage.getPreservedSet();
-      for (std::map<AnalysisID, Pass*>::iterator I = CurrentAnalyses.begin(),
-             E = CurrentAnalyses.end(); I != E; ) {
-        if (std::find(PreservedSet.begin(), PreservedSet.end(), I->first) ==
-            PreservedSet.end()) {             // Analysis not preserved!
-          CurrentAnalyses.erase(I);           // Remove from available analyses
-          I = CurrentAnalyses.begin();
-        } else {
-          ++I;
-        }
-      }
-    }
-
-    // Add this pass to the currently available set...
-    if (const PassInfo *PI = P->getPassInfo()) {
-      CurrentAnalyses[PI] = P;
-
-      // This pass is the current implementation of all of the interfaces it
-      // implements as well.
-      //
-      const std::vector<const PassInfo*> &II = PI->getInterfacesImplemented();
-      for (unsigned i = 0, e = II.size(); i != e; ++i)
-        CurrentAnalyses[II[i]] = P;
-    }
-
+    removeNonPreservedAnalyses(AnUsage);
+    
+    makeCurrentlyAvailable(P);
+    
     // For now assume that our results are never used...
     LastUseOf[P] = P;
   }
-
+  
   // For FunctionPass subclasses, we must be sure to batch the FunctionPass's
   // together in a BatcherClass object so that all of the analyses are run
   // together a function at a time.
   //
   void addPass(SubPassClass *MP, AnalysisUsage &AnUsage) {
-    if (Batcher == 0) // If we don't have a batcher yet, make one now.
-      Batcher = new BatcherClass(this);
+
+    if (Batcher == 0) { // If we don't have a batcher yet, make one now.
+      assert(dynamic_cast<PMType*>(this) && 
+             "It wasn't the PassManager type I thought it was");
+      Batcher = new BatcherClass((static_cast<PMType*>(this)));
+    }
+
     // The Batcher will queue the passes up
     MP->addToPassManager(Batcher, AnUsage);
   }
@@ -579,6 +504,18 @@ private:
     }
   }
 
+  void addRequiredPasses(const std::vector<AnalysisID> &Required) {
+    for (std::vector<AnalysisID>::const_iterator I = Required.begin(),
+         E = Required.end(); I != E; ++I) {
+      if (getAnalysisOrNullDown(*I) == 0) {
+        Pass *AP = (*I)->createPass();
+        if (ImmutablePass *IP = dynamic_cast<ImmutablePass *> (AP)) add(IP);
+        else if (PassClass *RP = dynamic_cast<PassClass *> (AP)) add(RP);
+        else assert (0 && "Wrong kind of pass for this PassManager");
+      }
+    }
+  }
+
 public:
   // When an ImmutablePass is added, it gets added to the top level pass
   // manager.
@@ -620,201 +557,352 @@ public:
     // Initialize the immutable pass...
     IP->initializePass();
   }
+private:
+  
+  // Add any immutable passes to the CurrentAnalyses set...
+  inline void addImmutablePasses() { 
+    for (unsigned i = 0, e = ImmutablePasses.size(); i != e; ++i) {
+      ImmutablePass *IPass = ImmutablePasses[i];
+      if (const PassInfo *PI = IPass->getPassInfo()) {
+        CurrentAnalyses[PI] = IPass;
+
+        const std::vector<const PassInfo*> &II = PI->getInterfacesImplemented();
+        for (unsigned i = 0, e = II.size(); i != e; ++i)
+          CurrentAnalyses[II[i]] = IPass;
+      }
+    }
+  }
+  
+  // Run all of the passes
+  inline bool runPasses(UnitType *M, 
+                 std::map<Pass *, std::vector<Pass*> > &LastUserOf) { 
+    bool MadeChanges = false;
+    
+    for (unsigned i = 0, e = Passes.size(); i < e; ++i) {
+      PassClass *P = Passes[i];
+
+      PMDebug::PrintPassInformation(getDepth(), "Executing Pass", P, M);
+
+      // Get information about what analyses the pass uses...
+      AnalysisUsage AnUsage;
+      P->getAnalysisUsage(AnUsage);
+      PMDebug::PrintAnalysisSetInfo(getDepth(), "Required", P,
+                                    AnUsage.getRequiredSet());
+      
+      initialiseAnalysisImpl(P, AnUsage);
+      
+      // Run the sub pass!
+      if (TheTimeInfo) TheTimeInfo->passStarted(P);
+      bool Changed = runPass(P, M);
+      if (TheTimeInfo) TheTimeInfo->passEnded(P);
+      MadeChanges |= Changed;
+
+      // Check for memory leaks by the pass...
+      LeakDetector::checkForGarbage(std::string("after running pass '") +
+                                    P->getPassName() + "'");
+
+      if (Changed)
+        PMDebug::PrintPassInformation(getDepth()+1, "Made Modification", P, M);
+      PMDebug::PrintAnalysisSetInfo(getDepth(), "Preserved", P,
+                                    AnUsage.getPreservedSet());
+      
+      // Erase all analyses not in the preserved set
+      removeNonPreservedAnalyses(AnUsage);
+      
+      makeCurrentlyAvailable(P);
+      
+      // free memory and remove dead passes from the CurrentAnalyses list...
+      removeDeadPasses(P, M, LastUserOf);
+    }
+    
+    return MadeChanges;
+  }
+  
+  // All Required analyses should be available to the pass as it runs!  Here
+  // we fill in the AnalysisImpls member of the pass so that it can
+  // successfully use the getAnalysis() method to retrieve the
+  // implementations it needs.
+  //
+  inline void initialiseAnalysisImpl(PassClass *P, AnalysisUsage &AnUsage) { 
+    P->AnalysisImpls.clear();
+    P->AnalysisImpls.reserve(AnUsage.getRequiredSet().size());
+    
+    for (std::vector<const PassInfo *>::const_iterator
+         I = AnUsage.getRequiredSet().begin(),
+         E = AnUsage.getRequiredSet().end(); I != E; ++I) {
+      Pass *Impl = getAnalysisOrNullUp(*I);
+      if (Impl == 0) {
+        std::cerr << "Analysis '" << (*I)->getPassName()
+                  << "' used but not available!";
+        assert(0 && "Analysis used but not available!");
+      } else if (PassDebugging == Details) {
+        if ((*I)->getPassName() != std::string(Impl->getPassName()))
+          std::cerr << "    Interface '" << (*I)->getPassName()
+                  << "' implemented by '" << Impl->getPassName() << "'\n";
+      }
+      
+      P->AnalysisImpls.push_back(std::make_pair(*I, Impl));
+    }
+  }
+  
+  inline void removeNonPreservedAnalyses(AnalysisUsage &AnUsage) { 
+    if (!AnUsage.getPreservesAll()) {
+      const std::vector<AnalysisID> &PreservedSet = AnUsage.getPreservedSet();
+      for (std::map<AnalysisID, Pass*>::iterator I = CurrentAnalyses.begin(),
+           E = CurrentAnalyses.end(); I != E; )
+        if (std::find(PreservedSet.begin(), PreservedSet.end(), I->first) !=
+            PreservedSet.end())
+          ++I; // This analysis is preserved, leave it in the available set...
+      else {
+        if (!dynamic_cast<ImmutablePass*>(I->second)) {
+          std::map<AnalysisID, Pass*>::iterator J = I++;
+          CurrentAnalyses.erase(J);   // Analysis not preserved!
+        } else {
+          ++I;
+        }
+      }
+    }
+  }
+  
+  inline void removeDeadPasses(Pass* P, UnitType *M, 
+              std::map<Pass *, std::vector<Pass*> > &LastUserOf) { 
+    std::vector<Pass*> &DeadPass = LastUserOf[P];
+    for (std::vector<Pass*>::iterator I = DeadPass.begin(),E = DeadPass.end();
+          I != E; ++I) {
+      PMDebug::PrintPassInformation(getDepth()+1, "Freeing Pass", *I, M);
+      (*I)->releaseMemory();
+    }
+    
+    for (std::map<AnalysisID, Pass*>::iterator I = CurrentAnalyses.begin();
+          I != CurrentAnalyses.end(); ) {
+      std::vector<Pass*>::iterator DPI = std::find(DeadPass.begin(),
+                                                    DeadPass.end(), I->second);
+      if (DPI != DeadPass.end()) {    // This pass is dead now... remove it
+        std::map<AnalysisID, Pass*>::iterator IDead = I++;
+        CurrentAnalyses.erase(IDead);
+      } else {
+        ++I;  // Move on to the next element...
+      }
+    }
+  }
+  
+  inline void makeCurrentlyAvailable(Pass* P) { 
+    if (const PassInfo *PI = P->getPassInfo()) {
+      CurrentAnalyses[PI] = P;
+
+      // This pass is the current implementation of all of the interfaces it
+      // implements as well.
+      //
+      const std::vector<const PassInfo*> &II = PI->getInterfacesImplemented();
+      for (unsigned i = 0, e = II.size(); i != e; ++i)
+        CurrentAnalyses[II[i]] = P;
+    }
+  }
 };
 
 
 
 //===----------------------------------------------------------------------===//
-// PassManagerTraits<BasicBlock> Specialization
+// BasicBlockPassManager
 //
 // This pass manager is used to group together all of the BasicBlockPass's
 // into a single unit.
 //
-template<> class PassManagerTraits<BasicBlock> : public BasicBlockPass {
+class BasicBlockPassManager : public BasicBlockPass, 
+                              public BBTraits, 
+                              public PassManagerT<BBTraits> {
 public:
-  // PassClass - The type of passes tracked by this PassManager
-  typedef BasicBlockPass PassClass;
-
-  // SubPassClass - The types of classes that should be collated together
-  // This is impossible to match, so BasicBlock instantiations of PassManagerT
-  // do not collate.
-  //
-  typedef PassManagerT<Module> SubPassClass;
-
-  // BatcherClass - The type to use for collation of subtypes... This class is
-  // never instantiated for the PassManager<BasicBlock>, but it must be an
-  // instance of PassClass to typecheck.
-  //
-  typedef PassClass BatcherClass;
-
-  // ParentClass - The type of the parent PassManager...
-  typedef PassManagerT<Function> ParentClass;
-
-  // PMType - The type of the passmanager that subclasses this class
-  typedef PassManagerT<BasicBlock> PMType;
-
+  BasicBlockPassManager(BBTraits::ParentClass* PC) : 
+    PassManagerT<BBTraits>(PC) {
+  }
+  
+  BasicBlockPassManager(BasicBlockPassManager* BBPM) : 
+    PassManagerT<BBTraits>(BBPM->Parent) {
+  }
+  
   // runPass - Specify how the pass should be run on the UnitType
-  static bool runPass(PassClass *P, BasicBlock *M) {
-    // todo, init and finalize
+  virtual bool runPass(BBTraits::PassClass *P, BasicBlock *M) {
+    // TODO: init and finalize
     return P->runOnBasicBlock(*M);
   }
-
+  
+  virtual ~BasicBlockPassManager() {}
+  
+  virtual void dumpPassStructure(unsigned Offset = 0) { 
+    PassManagerT<BBTraits>::dumpPassStructure(Offset);
+  }
+  
   // getPMName() - Return the name of the unit the PassManager operates on for
   // debugging.
-  const char *getPMName() const { return "BasicBlock"; }
+  virtual const char *getPMName() const { return "BasicBlock"; }
+  
   virtual const char *getPassName() const { return "BasicBlock Pass Manager"; }
-
-  // Implement the BasicBlockPass interface...
+  
   virtual bool doInitialization(Module &M);
   virtual bool doInitialization(Function &F);
   virtual bool runOnBasicBlock(BasicBlock &BB);
   virtual bool doFinalization(Function &F);
   virtual bool doFinalization(Module &M);
-
+  
   virtual void getAnalysisUsage(AnalysisUsage &AU) const {
     AU.setPreservesAll();
   }
 };
 
-
-
 //===----------------------------------------------------------------------===//
-// PassManagerTraits<Function> Specialization
+// FunctionPassManager
 //
 // This pass manager is used to group together all of the FunctionPass's
 // into a single unit.
 //
-template<> class PassManagerTraits<Function> : public FunctionPass {
+class FunctionPassManagerT : public FunctionPass, 
+                             public FTraits, 
+                             public PassManagerT<FTraits> {
 public:
-  // PassClass - The type of passes tracked by this PassManager
-  typedef FunctionPass PassClass;
-
-  // SubPassClass - The types of classes that should be collated together
-  typedef BasicBlockPass SubPassClass;
-
-  // BatcherClass - The type to use for collation of subtypes...
-  typedef PassManagerT<BasicBlock> BatcherClass;
-
-  // ParentClass - The type of the parent PassManager...
-  typedef PassManagerT<Module> ParentClass;
-
-  // PMType - The type of the passmanager that subclasses this class
-  typedef PassManagerT<Function> PMType;
-
-  // runPass - Specify how the pass should be run on the UnitType
-  static bool runPass(PassClass *P, Function *F) {
-    return P->runOnFunction(*F);
+  FunctionPassManagerT() : PassManagerT<FTraits>(0) {}
+  
+  // Parent constructor
+  FunctionPassManagerT(FTraits::ParentClass* PC) : PassManagerT<FTraits>(PC) {}
+  
+  FunctionPassManagerT(FunctionPassManagerT* FPM) : 
+    PassManagerT<FTraits>(FPM->Parent) {
   }
-
+  
+  virtual ~FunctionPassManagerT() {}
+  
+  virtual void dumpPassStructure(unsigned Offset = 0) { 
+    PassManagerT<FTraits>::dumpPassStructure(Offset);
+  }
+  
   // getPMName() - Return the name of the unit the PassManager operates on for
   // debugging.
-  const char *getPMName() const { return "Function"; }
+  virtual const char *getPMName() const { return "Function"; }
+  
   virtual const char *getPassName() const { return "Function Pass Manager"; }
-
-  // Implement the FunctionPass interface...
-  virtual bool doInitialization(Module &M);
+  
   virtual bool runOnFunction(Function &F);
+  
+  virtual bool doInitialization(Module &M);
+  
   virtual bool doFinalization(Module &M);
-
+  
   virtual void getAnalysisUsage(AnalysisUsage &AU) const {
     AU.setPreservesAll();
   }
-};
-
-
-
-//===----------------------------------------------------------------------===//
-// PassManagerTraits<Module> Specialization
-//
-// This is the top level PassManager implementation that holds generic passes.
-//
-template<> class PassManagerTraits<Module> : public ModulePass {
-public:
-  // PassClass - The type of passes tracked by this PassManager
-  typedef ModulePass PassClass;
-
-  // SubPassClass - The types of classes that should be collated together
-  typedef FunctionPass SubPassClass;
-
-  // BatcherClass - The type to use for collation of subtypes...
-  typedef PassManagerT<Function> BatcherClass;
-
-  // ParentClass - The type of the parent PassManager...
-  typedef AnalysisResolver ParentClass;
-
+  
   // runPass - Specify how the pass should be run on the UnitType
-  static bool runPass(PassClass *P, Module *M) { return P->runOnModule(*M); }
-
-  // getPMName() - Return the name of the unit the PassManager operates on for
-  // debugging.
-  const char *getPMName() const { return "Module"; }
-  virtual const char *getPassName() const { return "Module Pass Manager"; }
-
-  // runOnModule - Implement the PassManager interface.
-  bool runOnModule(Module &M) {
-    return ((PassManagerT<Module>*)this)->runOnUnit(&M);
+  virtual bool runPass(FTraits::PassClass *P, Function *F) {
+    return P->runOnFunction(*F);
   }
 };
 
 
+//===----------------------------------------------------------------------===//
+// ModulePassManager
+//
+// This is the top level PassManager implementation that holds generic passes.
+//
+class ModulePassManager : public ModulePass, 
+                          public MTraits, 
+                          public PassManagerT<MTraits> {
+public:
+  ModulePassManager() : PassManagerT<MTraits>(0) {}
+  
+  // Batcher Constructor
+  ModulePassManager(MTraits::ParentClass* PC) : PassManagerT<MTraits>(PC) {}
+  
+  ModulePassManager(ModulePassManager* MPM) : 
+    PassManagerT<MTraits>((MPM->Parent)) {
+  }
+  
+  virtual ~ModulePassManager() {}
+  
+  virtual void dumpPassStructure(unsigned Offset = 0) { 
+    PassManagerT<MTraits>::dumpPassStructure(Offset);
+  }
+  
+  // getPMName() - Return the name of the unit the PassManager operates on for
+  // debugging.
+  virtual const char *getPassName() const { return "Module Pass Manager"; }
+  
+  // getPMName() - Return the name of the unit the PassManager operates on for
+  // debugging.
+  virtual const char *getPMName() const { return "Module"; }
+  
+  // runOnModule - Implement the PassManager interface.
+  virtual bool runOnModule(Module &M);
+  
+  // runPass - Specify how the pass should be run on the UnitType
+  virtual bool runPass(MTraits::PassClass *P, Module *M) {
+    return P->runOnModule(*M);
+  }
+};
 
 //===----------------------------------------------------------------------===//
-// PassManagerTraits Method Implementations
+// PassManager Method Implementations
 //
 
-// PassManagerTraits<BasicBlock> Implementations
+// BasicBlockPassManager Implementations
 //
-inline bool PassManagerTraits<BasicBlock>::doInitialization(Module &M) {
+
+inline bool BasicBlockPassManager::runOnBasicBlock(BasicBlock &BB) {
+  return ((BBTraits::PMType*)this)->runOnUnit(&BB);
+}
+
+inline bool BasicBlockPassManager::doInitialization(Module &M) {
   bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doInitialization(M);
+  for (unsigned i = 0, e =((BBTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((BBTraits::PMType*)this)->Passes[i]->doInitialization(M);
   return Changed;
 }
 
-inline bool PassManagerTraits<BasicBlock>::doInitialization(Function &F) {
+inline bool BasicBlockPassManager::doInitialization(Function &F) {
   bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doInitialization(F);
+  for (unsigned i = 0, e =((BBTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((BBTraits::PMType*)this)->Passes[i]->doInitialization(F);
   return Changed;
 }
 
-inline bool PassManagerTraits<BasicBlock>::runOnBasicBlock(BasicBlock &BB) {
-  return ((PMType*)this)->runOnUnit(&BB);
-}
-
-inline bool PassManagerTraits<BasicBlock>::doFinalization(Function &F) {
+inline bool BasicBlockPassManager::doFinalization(Function &F) {
   bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doFinalization(F);
+  for (unsigned i = 0, e =((BBTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((BBTraits::PMType*)this)->Passes[i]->doFinalization(F);
   return Changed;
 }
 
-inline bool PassManagerTraits<BasicBlock>::doFinalization(Module &M) {
+inline bool BasicBlockPassManager::doFinalization(Module &M) {
   bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doFinalization(M);
+  for (unsigned i=0, e = ((BBTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((BBTraits::PMType*)this)->Passes[i]->doFinalization(M);
   return Changed;
 }
 
-
-// PassManagerTraits<Function> Implementations
+// FunctionPassManagerT Implementations
 //
-inline bool PassManagerTraits<Function>::doInitialization(Module &M) {
+
+inline bool FunctionPassManagerT::runOnFunction(Function &F) {
+  return ((FTraits::PMType*)this)->runOnUnit(&F);
+}
+
+inline bool FunctionPassManagerT::doInitialization(Module &M) {
   bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doInitialization(M);
+  for (unsigned i=0, e = ((FTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((FTraits::PMType*)this)->Passes[i]->doInitialization(M);
   return Changed;
 }
 
-inline bool PassManagerTraits<Function>::runOnFunction(Function &F) {
-  return ((PMType*)this)->runOnUnit(&F);
+inline bool FunctionPassManagerT::doFinalization(Module &M) {
+  bool Changed = false;
+  for (unsigned i=0, e = ((FTraits::PMType*)this)->Passes.size(); i != e; ++i)
+    ((FTraits::PMType*)this)->Passes[i]->doFinalization(M);
+  return Changed;
 }
 
-inline bool PassManagerTraits<Function>::doFinalization(Module &M) {
-  bool Changed = false;
-  for (unsigned i = 0, e = ((PMType*)this)->Passes.size(); i != e; ++i)
-    ((PMType*)this)->Passes[i]->doFinalization(M);
-  return Changed;
+// ModulePassManager Implementations
+//
+
+bool ModulePassManager::runOnModule(Module &M) {
+  return ((PassManagerT<MTraits>*)this)->runOnUnit(&M);
 }
 
 } // End llvm namespace
