@@ -20,8 +20,8 @@
 #include "llvm/Bytecode/Reader.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/System/Signals.h"
-#include <cstdio>
 #include <iostream>
+#include <iomanip>
 #include <map>
 #include <set>
 
@@ -115,8 +115,8 @@ int main(int argc, char **argv) {
     std::string ErrorMessage;
     Module *M = ParseBytecodeFile(BytecodeFile, &ErrorMessage);
     if (M == 0) {
-      std::cerr << argv[0] << ": " << BytecodeFile << ": " << ErrorMessage
-                << "\n";
+      std::cerr << argv[0] << ": " << BytecodeFile << ": " 
+        << ErrorMessage << "\n";
       return 1;
     }
 
@@ -127,7 +127,7 @@ int main(int argc, char **argv) {
     std::map<const BasicBlock*, unsigned> BlockFreqs;
     std::map<ProfileInfoLoader::Edge, unsigned> EdgeFreqs;
 
-    // Output a report.  Eventually, there will be multiple reports selectable on
+    // Output a report. Eventually, there will be multiple reports selectable on
     // the command line, for now, just keep things simple.
 
     // Emit the most frequent function table...
@@ -136,7 +136,7 @@ int main(int argc, char **argv) {
     FuncFreqs.insert(FunctionCounts.begin(), FunctionCounts.end());
 
     // Sort by the frequency, backwards.
-    std::sort(FunctionCounts.begin(), FunctionCounts.end(),
+    sort(FunctionCounts.begin(), FunctionCounts.end(),
               PairSecondSortReverse<Function*>());
 
     unsigned long long TotalExecutions = 0;
@@ -158,16 +158,18 @@ int main(int argc, char **argv) {
     std::cout << "Function execution frequencies:\n\n";
 
     // Print out the function frequencies...
-    printf(" ##   Frequency\n");
+    std::cout << " ##   Frequency\n";
     for (unsigned i = 0, e = FunctionCounts.size(); i != e; ++i) {
       if (FunctionCounts[i].second == 0) {
-        printf("\n  NOTE: %d function%s never executed!\n",
-               e-i, e-i-1 ? "s were" : " was");
+        std::cout << "\n  NOTE: " << e-i << " function" <<
+               (e-i-1 ? "s were" : " was") << " never executed!\n";
         break;
       }
 
-      printf("%3d. %5u/%llu %s\n", i+1, FunctionCounts[i].second, TotalExecutions,
-             FunctionCounts[i].first->getName().c_str());
+      std::cout << std::setw(3) << i+1 << ". " 
+        << std::setw(5) << FunctionCounts[i].second << "/"
+        << TotalExecutions << " "
+        << FunctionCounts[i].first->getName().c_str() << "\n";
     }
 
     std::set<Function*> FunctionsToPrint;
@@ -183,23 +185,26 @@ int main(int argc, char **argv) {
         TotalExecutions += Counts[i].second;
 
       // Sort by the frequency, backwards.
-      std::sort(Counts.begin(), Counts.end(),
+      sort(Counts.begin(), Counts.end(),
                 PairSecondSortReverse<BasicBlock*>());
 
       std::cout << "\n===" << std::string(73, '-') << "===\n";
       std::cout << "Top 20 most frequently executed basic blocks:\n\n";
 
       // Print out the function frequencies...
-      printf(" ##      %%%% \tFrequency\n");
+      std::cout <<" ##      %% \tFrequency\n";
       unsigned BlocksToPrint = Counts.size();
       if (BlocksToPrint > 20) BlocksToPrint = 20;
       for (unsigned i = 0; i != BlocksToPrint; ++i) {
         if (Counts[i].second == 0) break;
         Function *F = Counts[i].first->getParent();
-        printf("%3d. %5.2f%% %5u/%llu\t%s() - %s\n", i+1,
-               Counts[i].second/(double)TotalExecutions*100,
-               Counts[i].second, TotalExecutions,
-               F->getName().c_str(), Counts[i].first->getName().c_str());
+        std::cout << std::setw(3) << i+1 << ". " 
+          << std::setw(5) << std::setprecision(2) 
+          << Counts[i].second/(double)TotalExecutions*100 << "% "
+          << std::setw(5) << Counts[i].second << "/"
+          << TotalExecutions << "\t"
+          << F->getName().c_str() << "() - "
+          << Counts[i].first->getName().c_str() << "\n";
         FunctionsToPrint.insert(F);
       }
 
