@@ -22,7 +22,6 @@
 #  -nodejagnu       Do not run feature or regression tests
 #  -parallel        Run two parallel jobs with GNU Make.
 #  -release         Build an LLVM Release version
-#  -pedantic        Enable additional GCC warnings to detect possible errors.
 #  -enable-llcbeta  Enable testing of beta features in llc.
 #  -disable-llc     Disable LLC tests in the nightly tester.
 #  -disable-jit     Disable JIT tests in the nightly tester.
@@ -37,6 +36,12 @@
 #  -cvstag          Check out a specific CVS tag to build LLVM (useful for
 #                   testing release branches)
 #  -target          Specify the target triplet
+#  -cflags          Next argument specifies that C compilation options that
+#                   override the default.
+#  -cxxflags        Next argument specifies that C++ compilation options that
+#                   override the default.
+#  -ldflags         Next argument specifies that linker options that override
+#                   the default.
 #
 #  ---------------- Options to configure llvm-test ----------------------------
 #  -spec2000path    Path to the benchspec directory in the SPEC 2000 distro
@@ -146,7 +151,21 @@ sub AddRecord {
 
 sub AddPreTag {  # Add pre tags around nonempty list, or convert to "none"
   $_ = shift;
-  if (length) { return "<ul><pre>$_</pre></ul>"; } else { "<b>none</b><br>"; }
+  if (length) { return "<pre>$_</pre>"; } else { "<b>none</b><br>"; }
+}
+
+sub ArrayToList { # Add <li> tags around nonempty list or convert to "none"
+  my $result = "";
+  if (scalar @_) {
+    $result = "<ul>";
+    foreach $item (@_) {
+      $result .= "<li><tt>$item</tt></li>";
+    }
+    $result .= "</ul>";
+  } else {
+    $result = "<p><b>none</b></p>";
+  }
+  return $result;
 }
 
 sub ChangeDir { # directory, logical name
@@ -280,10 +299,6 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
   if (/^-norunningtests$/) { $NORUNNINGTESTS = 1; next; }
   if (/^-parallel$/)       { $MAKEOPTS = "$MAKEOPTS -j2 -l3.0"; next; }
   if (/^-release$/)        { $MAKEOPTS = "$MAKEOPTS ENABLE_OPTIMIZED=1"; next; }
-  if (/^-pedantic$/)       { 
-      $MAKEOPTS = "$MAKEOPTS CompileOptimizeOpts='-O3 -DNDEBUG -finline-functions -Wpointer-arith -Wcast-align -Wno-deprecated -Wold-style-cast -Wabi -Woverloaded-virtual -ffor-scope'"; 
-      next; 
-  }
   if (/^-enable-llcbeta$/) { $PROGTESTOPTS .= " ENABLE_LLCBETA=1"; next; }
   if (/^-disable-llc$/)    { $PROGTESTOPTS .= " DISABLE_LLC=1";
                              $CONFIGUREARGS .= " --disable-llc_diffs"; next; }
@@ -304,6 +319,15 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
   if (/^-target/)          {
     $CONFIGUREARGS .= " --target=$ARGV[0]"; shift; next;
   }
+  if (/^-cflags/)          {
+    $MAKEOPTS = "$MAKEOPTS C.Flags=\'$ARGV[0]\'"; shift; next;
+  }
+  if (/^-cxxflags/)        {
+    $MAKEOPTS = "$MAKEOPTS CXX.Flags=\'$ARGV[0]\'"; shift; next;
+  }
+  if (/^-ldflags/)         {
+    $MAKEOPTS = "$MAKEOPTS LD.Flags=\'$ARGV[0]\'"; shift; next;
+  }
   if (/^-noexternals$/)    { $NOEXTERNALS = 1; next; }
   if (/^-nodejagnu$/)      { $NODEJAGNU = 1; next; }
   if (/^-spec2000path$/)   {
@@ -462,7 +486,7 @@ if (`grep '^gmake[^:]*: .*Error' $BuildLog | wc -l` + 0 ||
   $BuildStatus = "<h3><font color='red'>error: compilation " .
                 "<a href=\"$DATE-Build-Log.txt\">aborted</a></font></h3>";
   $BuildError = 1;
-  if ($VERBOSE) { print "BUILD ERROR\n"; }
+  print "\n***ERROR BUILDING TREE\n\n";
 }
 
 if ($BuildError) { $NODEJAGNU=1; }
@@ -518,7 +542,7 @@ foreach $Warning (@Warn) {
   }
 }
 my $WarningsFile =  join "\n", @Warnings;
-my $WarningsList = AddPreTag $WarningsFile;
+my $WarningsList = ArrayToList @Warnings;
 $WarningsFile =~ s/:[0-9]+:/::/g;
 
 # Emit the warnings file, so we can diff...
@@ -529,8 +553,10 @@ my ($WarningsAdded, $WarningsRemoved) = DiffFiles "-Warnings.txt";
 print "ADDED   WARNINGS:\n$WarningsAdded\n\n" if (length $WarningsAdded);
 print "REMOVED WARNINGS:\n$WarningsRemoved\n\n" if (length $WarningsRemoved);
 
-$WarningsAdded = AddPreTag $WarningsAdded;
-$WarningsRemoved = AddPreTag $WarningsRemoved;
+my @TmpWarningsAdded = split "\n", $WarningsAdded;
+my @TmpWarningsRemoved = split "\n", $WarningsRemoved;
+$WarningsAdded = ArrayToList @TmpWarningsAdded;
+$WarningsRemoved = ArrayToList @TmpWarningsRemoved;
 
 #
 # Get some statistics about CVS commits over the current day...
@@ -847,7 +873,10 @@ my $TestFinishTime = gmtime() . " GMT<br>" . localtime() . " (local)";
 my $TestPlatform = `uname -a`;
 eval "\$Output = <<ENDOFFILE;$TemplateContents\nENDOFFILE\n";
 WriteFile "$DATE.html", $Output;
-system ( "ln -sf $DATE.html index.html" );
+
+# Remove the symlink before creating it for systems that don't have "ln -sf".
+system ("rm index.html");
+system ("ln -s $DATE.html index.html");
 
 # Change the index.html symlink...
 
